import qrunner
from qrunner import story, title


class PatentPage(qrunner.Page):
    url = None
    LOC_SEARCH_INPUT = {'id_': 'driver-home-step1', 'desc': '查专利首页输入框'}
    LOC_SEARCH_SUBMIT = {'id_': 'driver-home-step2', 'desc': '查专利首页搜索确认按钮'}
    
    def simple_search(self):
        self.elem(self.LOC_SEARCH_INPUT).set_text('无人机')
        self.elem(self.LOC_SEARCH_SUBMIT).click()


@story('专利检索')
class TestClass(qrunner.TestCase):
    
    def start(self):
        self.pp = PatentPage(self.driver)
    
    @title('专利简单检索')
    def testcase(self):
        self.pp.open()
        self.pp.simple_search()
        self.assertTitle('无人机专利检索-企知道')


if __name__ == '__main__':
    qrunner.main(
        platform='web',
        base_url='https://patents.qizhidao.com/'
    )



import re
import time
from typing import Union

import jmespath

from qrunner.core.api.request import ResponseResult, HttpRequest
from qrunner.utils.decorate import step
from qrunner.core.web.driver import WebDriver
from qrunner.core.web.element import WebElement
from qrunner.core.h5.driver import H5Driver
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.utils.log import logger
from qrunner.utils.config import config
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement


class TestCase(HttpRequest):
    """
    测试用例基类，所有测试用例需要继承该类
    """
    driver: Union[AndroidDriver, IosDriver, WebDriver, H5Driver] = None

    # ---------------------初始化-------------------------------
    def start_class(self):
        """
        Hook method for setup_class fixture
        :return:
        """
        pass

    def end_class(self):
        """
        Hook method for teardown_class fixture
        :return:
        """
        pass

    @classmethod
    def setup_class(cls):
        # 初始化driver
        platform = config.get_platform()
        logger.info(platform)
        if platform == 'android':
            serial_no = config.get_device()
            cls.driver = AndroidDriver(serial_no)
        elif platform == 'web':
            browser = config.get_browser()
            cls.driver = WebDriver(browser)
        elif platform == 'ios':
            serial_no = config.get_device()
            cls.driver = IosDriver(serial_no)
        cls().start_class()

    @classmethod
    def teardown_class(cls):
        if isinstance(cls().driver, WebDriver):
            cls().driver.quit()
        cls().end_class()

    def start(self):
        """
        Hook method for setup_method fixture
        :return:
        """
        pass

    def end(self):
        """
        Hook method for teardown_method fixture
        :return:
        """
        pass

    def setup_method(self):
        self.start_time = time.time()
        logger.debug(f"[start_time]: {time.strftime('%Y-%m-%d %H:%M:%S')}")
        # 启动应用
        # self.driver.force_start_app()
        self.start()

    def teardown_method(self):
        self.end()
        # self.driver.screenshot('用例执行完成截图')
        # self.screenshot('用例执行完成截图')
        # 退出应用
        # self.driver.stop_app()
        logger.debug(f"[end_time]: {time.strftime('%Y-%m-%d %H:%M:%S')}")
        take_time = time.time() - self.start_time
        logger.debug("[run_time]: {:.2f} s".format(take_time))

    # 公共方法
    @staticmethod
    def sleep(n: int):
        """休眠"""
        logger.debug(f'等待: {n}s')
        time.sleep(n)

    def screenshot(self, file_name):
        """截图"""
        self.driver.screenshot(file_name)

    def elem(self, **kwargs):
        if isinstance(self.driver, AndroidDriver):
            return AndroidElement(self.driver, **kwargs)
        elif isinstance(self.driver, WebDriver):
            return WebElement(self.driver, **kwargs)
        elif isinstance(self.driver, IosDriver):
            return IosElement(self.driver, **kwargs)

    def assertText(self, expect_value, timeout=5):
        """断言页面包含文本"""
        for _ in range(timeout + 1):
            try:
                page_source = self.driver.get_page_content()
                assert expect_value in page_source, f'页面内容不包含 {expect_value}'
                break
            except AssertionError:
                time.sleep(1)
        else:
            page_source = self.driver.get_page_content()
            assert expect_value in page_source, f'页面内容不包含 {expect_value}'

    def assertNotText(self, expect_value, timeout=5):
        """断言页面不包含文本"""
        for _ in range(timeout + 1):
            try:
                page_source = self.driver.get_page_content()
                assert expect_value not in page_source, f'页面内容不包含 {expect_value}'
                break
            except AssertionError:
                time.sleep(1)
        else:
            page_source = self.driver.get_page_content()
            assert expect_value not in page_source, f'页面内容仍然包含 {expect_value}'

    def assertElement(self, timeout=5, **kwargs):
        """断言元素存在"""
        for _ in range(timeout + 1):
            try:
                element = self.elem(**kwargs)
                assert element.exists(), f'元素 {kwargs} 不存在'
                break
            except AssertionError:
                time.sleep(1)
        else:
            assert self.elem(**kwargs).exists(), f'元素 {kwargs} 不存在'

    def assertNotElement(self, timeout=5, **kwargs):
        """断言元素不存在"""
        for _ in range(timeout + 1):
            try:
                assert not self.elem(**kwargs).exists(), f'元素 {kwargs} 仍然存在'
                break
            except AssertionError:
                time.sleep(1)
        else:
            assert not self.elem(**kwargs).exists(), f'元素 {kwargs} 仍然存在'

    # WEB专用方法
    def switch_frame(self, frame_id: str):
        """切换到iframe中"""
        self.driver.switch_to_frame(frame_id)

    def frame_out(self):
        """从iframe中回到顶层页面"""
        self.driver.switch_to_frame_out()

    def exe_js(self, script: str, *args):
        """执行js语句"""
        self.driver.execute_js(script, *args)

    def click_js(self, **kwargs):
        """正常点击方法失败时使用"""
        elem = self.elem(**kwargs).get_element()
        self.driver.execute_js("$(arguments[0]).click()", elem)

    def assertTitle(self, expect_value=None, timeout=5):
        """断言页面标题等于"""
        for _ in range(timeout + 1):
            try:
                title = self.driver.get_title()
                assert expect_value == title, f'页面标题 {title} 不等于 {expect_value}'
                break
            except AssertionError:
                time.sleep(1)
        else:
            title = self.driver.get_title()
            assert expect_value == title, f'页面标题 {title} 不等于 {expect_value}'

    def assertInTitle(self, expect_value=None, timeout=5):
        """断言页面标题包含"""
        for _ in range(timeout + 1):
            try:
                title = self.driver.get_title()
                assert expect_value in title, f'页面标题 {title} 不包含 {expect_value}'
                break
            except AssertionError:
                time.sleep(1)
        else:
            title = self.driver.get_title()
            assert expect_value in title, f'页面标题 {title} 不包含 {expect_value}'

    def assertUrl(self, expect_value=None, timeout=5):
        """断言页面url等于"""
        for _ in range(timeout + 1):
            try:
                url = self.driver.get_url()
                assert expect_value == url, f'页面url {url} 不等于 {expect_value}'
                break
            except AssertionError:
                time.sleep(1)
        else:
            url = self.driver.get_url()
            assert expect_value == url, f'页面url {url} 不等于 {expect_value}'

    def assertInUrl(self, expect_value=None, timeout=5):
        """断言页面url包含"""
        for _ in range(timeout + 1):
            try:
                url = self.driver.get_url()
                assert expect_value in url, f'页面url {url} 不包含 {expect_value}'
                break
            except AssertionError:
                time.sleep(1)
        else:
            url = self.driver.get_url()
            assert expect_value in url, f'页面url {url} 不包含 {expect_value}'

    def assertAlertText(self, expect_value):
        """断言弹窗文本"""
        alert_text = self.driver.get_alert_text()
        assert expect_value == alert_text, f'弹窗文本 {alert_text} 等于 {expect_value}'

    # APP专用方法
    def install_app(self, url):
        """安装应用"""
        self.driver.install_app(url)

    def new_install_app(self, url, pkg_name=None):
        """先卸载再安装应用"""
        self.driver.new_install_app(url, pkg_name)

    def uninstall_app(self, pkg=None):
        """卸载应用"""
        self.driver.uninstall_app(pkg)

    def start_app(self, pkg=None):
        """强制启动应用"""
        self.driver.force_start_app(pkg)

    def stop_app(self, pkg=None):
        """停止应用"""
        self.driver.stop_app(pkg)

    # API专用方法
    @staticmethod
    def assertStatusCode(status_code):
        """
        断言状态码
        """
        with step(f'断言响应状态码 == {status_code}'):
            actual_code = ResponseResult.status_code
            logger.info(f'断言 {actual_code} 等于 {status_code}')
            assert actual_code == status_code, \
                f'status_code {ResponseResult} != {status_code}'

    @staticmethod
    def assertPath(path, value):
        """
        功能同assertEq，用于兼容历史代码
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} == {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 等于 {value}')
            assert search_value == value, f'{search_value} != {value}'

    @staticmethod
    def assertNotEq(path, value):
        """
        值不等于
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} != {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 不等于 {value}')
            assert search_value != value, f"{search_value} 等于 {value}"

    @staticmethod
    def assertLenEq(path, value):
        """
        断言列表长度等于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 的长度 == {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 的长度等于 {value}')
            assert len(search_value) == value, f"{search_value} 的长度不等于 {value}"

    @staticmethod
    def assertLenGt(path, value):
        """
        断言列表长度大于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 的长度 > {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 的长度大于 {value}')
            assert len(search_value) > value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenGtOrEq(path, value):
        """
        断言列表长度大于等于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 的长度 >= {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 的长度大于等于 {value}')
            assert len(search_value) >= value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenLt(path, value):
        """
        断言列表长度小于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 的长度 < {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 的长度小于 {value}')
            assert len(search_value) < value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertLenLtOrEq(path, value):
        """
        断言列表长度小于等于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 的长度 <= {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 的长度小于等于 {value}')
            assert len(search_value) <= value, f"{search_value} 的长度不大于 {value}"

    @staticmethod
    def assertGt(path, value):
        """
        值大于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} > {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 大于 {value}')
            assert int(search_value) > int(value), f"{search_value} 不大于 {value}"

    @staticmethod
    def assertGtOrEq(path, value):
        """
        值大于等于
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} >= {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            if isinstance(search_value, str):
                search_value = int(search_value)
            logger.info(f'断言 {search_value} 大于等于 {value}')
            assert search_value >= value, f"{search_value} 小于 {value}"

    @staticmethod
    def assertLt(path, value):
        """
        值小于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} < {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 小于 {value}')
            assert int(search_value) < int(value), f"{search_value} 不大于 {value}"

    @staticmethod
    def assertLtOrEq(path, value):
        """
        值小于等于多少
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} <= {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 小于等于 {value}')
            assert int(search_value) <= int(value), f"{search_value} 不大于 {value}"

    @staticmethod
    def assertRange(path, start: int, end: int):
        """值在(start, end)范围内
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 在({start}, {end})范围内'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 在 ({start}, {end}) 范围内')
            assert (int(search_value) > start) & (int(search_value) < end), f'{search_value} 不在({start}, {end})范围内'

    @staticmethod
    def assertIn(path, value):
        """
        断言匹配结果被value_list包含
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 被 {value} 包含'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 被 {value} 包含')
            assert search_value in value, f"{value} 不包含 {search_value}"

    @staticmethod
    def assertNotIn(path, value):
        """
        断言匹配结果不被value_list包含
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 不被 {value} 包含'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 不被 {value} 包含')
            assert search_value not in value, f"{value} 包含 {search_value}"

    @staticmethod
    def assertNotExists(path):
        """断言字段不存在"""
        with step(f'断言响应中 {path} 值为None或字段不存在'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 为None')
            assert search_value is None, f'仍然包含 {path} 为 {search_value}'

    @staticmethod
    def assertContains(path, value):
        """
        断言匹配结果包含value
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 包含 {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 包含 {value}')
            assert value in search_value, f"{search_value} 不包含 {value}"

    @staticmethod
    def assertNotContains(path, value):
        """
        断言匹配结果不包含value
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 不包含 {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 不包含 {value}')
            assert value not in search_value, f"{search_value} 包含 {value}"

    @staticmethod
    def assertTypeMatch(path, value_type):
        """
        类型匹配
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 的数据类型等于 {value_type}'):
            if not isinstance(value_type, type):
                if value_type == 'int':
                    value_type = int
                elif value_type == 'str':
                    value_type = str
                elif value_type == 'list':
                    value_type = list
                elif value_type == 'dict':
                    value_type = dict
                else:
                    value_type = str

            search_value = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 的数据类型等于 {value_type}')
            assert isinstance(search_value, value_type), f'{search_value} 不是 {value_type} 类型'

    @staticmethod
    def assertStartsWith(path, value):
        """
        以什么开头
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 以 {value} 开头'):
            search_value: str = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 以 {value} 开头')
            assert search_value.startswith(value), f'{search_value} 不以 {value} 开头'

    @staticmethod
    def assertEndsWith(path, value):
        """
        以什么结尾
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 以 {value} 结尾'):
            search_value: str = jmespath.search(path, ResponseResult.response)
            logger.info(f'断言 {search_value} 以 {value} 结尾')
            assert search_value.endswith(value), f'{search_value} 不以 {value} 结尾'

    @staticmethod
    def assertRegexMatch(path, value):
        """
        正则匹配
        doc: https://jmespath.org/
        """
        with step(f'断言响应中 {path} 正则匹配表达式 {value}'):
            search_value = jmespath.search(path, ResponseResult.response)
            match_obj = re.match(r'' + value, search_value, flags=re.I)
            logger.info(f'断言 {search_value} 匹配正则表达式 {value}')
            assert match_obj is not None, f'结果 {search_value} 匹配失败'



import os
import shutil
import subprocess
import time

import allure
import wda

from qrunner.core.ios.common import get_device_list, get_tcp_port
from qrunner.utils.log import logger
from qrunner.utils.config import config
from qrunner.utils.exceptions import DeviceNotFoundException, ScreenFailException


def _start_wda_xctest(udid: str, port, wda_bundle_id=None) -> bool:
    xctool_path = shutil.which("tidevice")
    logger.info(f"WDA is not running, exec: {xctool_path} -u {udid} wdaproxy --port {port} -B {wda_bundle_id}")
    args = []
    if udid:
        args.extend(['-u', udid])
    args.append('wdaproxy')
    args.extend(['--port', str(port)])
    if wda_bundle_id is not None:
        args.extend(['-B', wda_bundle_id])
    p = subprocess.Popen([xctool_path] + args)
    time.sleep(3)
    if p.poll() is not None:
        logger.warning("xctest launch failed")
        return False
    return True


class IosDriver(object):
    _instance = {}

    def __new__(cls, serial_no=None, bundle_id=None):
        if serial_no not in cls._instance:
            cls._instance[serial_no] = super().__new__(cls)
        return cls._instance[serial_no]

    def __init__(self, serial_no=None):
        self.serial_no = serial_no if serial_no else config.get_device()
        if serial_no in get_device_list():
            self.serial_no = serial_no
        else:
            raise DeviceNotFoundException(msg=f'设备 {serial_no} 未连接')
        self.bundle_id = config.get_pkg()
        logger.debug(self.bundle_id)
        logger.info(f'启动 ios driver for {self.serial_no}')
        # 启动wda
        port = get_tcp_port(self.serial_no)
        self.d = wda.Client(f"http://localhost:{port}")
        if not self.d.is_ready():
            logger.info('wda未启动，开始启动wda')
            _start_wda_xctest(self.serial_no, port=port)

    @classmethod
    def get_instance(cls, serial_no=None):
        if serial_no not in cls._instance:
            logger.info(f'{serial_no} Create ios driver singleton')
            return IosDriver(serial_no)
        return IosDriver._instance[serial_no]

    def install_app(self, ipa_url):
        """安装应用"""
        cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
        logger.info(f'安装应用: {ipa_url}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 安装应用{ipa_url} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')

    def new_install_app(self, ipa_url, bundle_id=None):
        """先卸载，再安装"""
        pkg_name = bundle_id if bundle_id else self.bundle_id
        self.uninstall_app(pkg_name)
        cmd = f'tidevice -u {self.serial_no} install {ipa_url}'
        logger.info(f'安装应用: {ipa_url}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 安装应用{ipa_url} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 安装应用{ipa_url}失败，因为{output}')

    def uninstall_app(self, bundle_id=None):
        """卸载应用"""
        pkg_name = bundle_id if bundle_id else self.bundle_id
        cmd = f'tidevice -u {self.serial_no} uninstall {pkg_name}'
        logger.info(f'卸载应用: {pkg_name}')
        output = subprocess.getoutput(cmd)
        if 'Complete' in output.split()[-1]:
            logger.info(f'{self.serial_no} 卸载应用{pkg_name} 成功')
            return
        else:
            logger.info(f'{self.serial_no} 卸载应用{pkg_name}失败，因为{output}')

    def start_app(self, bundle_id=None):
        """启动应用"""
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'启动应用: {bundle_id}')
        self.d.app_start(bundle_id)

    def force_start_app(self, bundle_id=None):
        """重启应用"""
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'强制启动应用: {bundle_id}')
        self.d.app_terminate(bundle_id)
        self.start_app(bundle_id)

    def stop_app(self, bundle_id=None):
        """停止应用"""
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'停止应用: {bundle_id}')
        self.d.app_terminate(bundle_id)

    def app_current(self):
        """获取运行中的app列表"""
        cur_apps = self.d.app_current()
        logger.info(f'获取运行中的app列表: {cur_apps}')
        return cur_apps

    def app_launch(self, bundle_id=None):
        """将应用切到前台"""
        if not bundle_id:
            bundle_id = self.bundle_id
        logger.info(f'将应用切到前台: {bundle_id}')
        self.d.app_launch(bundle_id)

    def back(self):
        """返回上一页"""
        logger.info('返回上一页')
        time.sleep(1)
        self.d.swipe(0, 100, 100, 100)

    def go_home(self):
        """返回手机主页"""
        logger.info('返回手机主页')
        self.d.home()

    def send_keys(self, value):
        """输入内容"""
        logger.info(f'输入: {value}')
        self.d.send_keys(value)

    def screenshot(self, file_name):
        """
        截图并保存到预定路径
        @param file_name: foo.png or fool
        @return:
        """
        # 把文件名处理成test.png的样式
        try:
            if '.' in file_name:
                file_name = file_name.split(r'.')[0]
            # 截图并保存到当前目录的images文件夹中
            img_dir = os.path.join(os.getcwd(), 'images')
            if os.path.exists(img_dir) is False:
                os.mkdir(img_dir)
            time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
            file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
            self.d.screenshot(file_path)
            # 上传allure报告
            allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
            return file_path
        except Exception as e:
            raise ScreenFailException(f'{file_name} 截图失败\n{str(e)}')

    # def screenshot_and_mark(self, file_name, rect):
    #     """给图片指定范围画上红框
    #     rect: [x, y, width, height]
    #     x: 左上坐标x
    #     y：左上角坐标y
    #     width：矩形宽度
    #     height：矩形高度
    #     """
    #     # 把文件名处理成test.png的样式
    #     if '.' in file_name:
    #         file_name = file_name.split(r'.')[0]
    #     # 截图并保存到当前目录的images文件夹中
    #     img_dir = os.path.join(os.getcwd(), 'images')
    #     if os.path.exists(img_dir) is False:
    #         os.mkdir(img_dir)
    #     time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
    #     file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
    #     self.d.screenshot(file_path)
    #     # 画框
    #     ImageRecognition.mark(file_path, rect)
    #     # 上传allure报告
    #     allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
    #     return file_path

    def get_page_content(self):
        """获取页面xml内容"""
        page_source = self.d.source(accessible=False)
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    def get_window_size(self):
        """获取屏幕尺寸"""
        size = self.d.window_size()
        logger.info(f'获取屏幕尺寸: {size}')
        return size

    def click(self, x, y):
        """点击坐标"""
        logger.info(f'点击坐标: ({x}, {y})')
        logger.info(f'{self.serial_no} Tap point ({x}, {y})')
        self.d.appium_settings({"snapshotMaxDepth": 0})
        self.d.tap(x, y)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(1)

    def double_click(self, x, y):
        """双击坐标"""
        logger.info(f'双击坐标: {x},{y}')
        self.d.double_tap(x, y)

    def tap_hold(self, x, y):
        """长按坐标"""
        logger.info(f'长按: {x},{y}')
        self.d.tap_hold(x, y, 1.0)

    def click_alert(self, alert_list: list):
        """点击弹窗"""
        try:
            self.d.alert.click(alert_list)
        except:
            pass

    def swipe(self, start_x, start_y, end_x, end_y, duration=0):
        """根据坐标滑动"""
        logger.info(f'从坐标({start_x}, {start_y})滑动到({end_x}, {end_y})')
        logger.info(f'{self.serial_no} swipe from point ({start_x}, {start_y}) to ({end_x}, {end_y})')
        self.d.appium_settings({"snapshotMaxDepth": 2})
        self.d.swipe(int(start_x), int(start_y), int(end_x), int(end_y), duration)
        self.d.appium_settings({"snapshotMaxDepth": 50})
        time.sleep(2)

    def swipe_by_screen_percent(self, start_x_percent, start_y_percent, end_x_percent, end_y_percent, duration=0):
        """根据屏幕百分比滑动"""
        logger.info(f'根据屏幕百分比进行滑动')
        w, h = self.d.window_size()
        start_x = w * start_x_percent
        start_y = h * start_y_percent
        end_x = w * end_x_percent
        end_y = h * end_y_percent
        self.swipe(start_x, start_y, end_x, end_y, duration=duration)

    def swipe_left(self, start_percent=1, end_percent=0.5):
        """往左滑动"""
        logger.info('往左边滑动')
        w, h = self.d.window_size()
        self.swipe(start_percent * (w - 1), h/2, end_percent * w, h/2)

    def swipe_right(self, start_percent=0.5, end_percent=1):
        """往右滑动"""
        logger.info('往右边滑动')
        w, h = self.d.window_size()
        self.swipe(start_percent * w, h / 2, end_percent * (w - 1), h / 2)

    def swipe_up(self, start_percent=0.8, end_percent=0.2):
        """往上滑动"""
        logger.info('往上边滑动')
        w, h = self.d.window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    def swipe_down(self, start_percent=0.2, end_percent=0.8):
        """往下滑动"""
        logger.info('往下面滑动')
        w, h = self.d.window_size()
        self.swipe(w / 2, start_percent * h, w / 2, end_percent * h)

    def check(self):
        """检查设备连接状态"""
        logger.info('健康检查')
        self.d.healthcheck()

    def locked(self):
        """检查手机是否锁屏"""
        logger.info('是否锁屏')
        status = self.d.locked()
        logger.info(status)
        return status

    def lock(self):
        """锁屏"""
        logger.info('锁住屏幕')
        self.d.lock()

    def unlock(self):
        """解锁"""
        logger.info('解锁屏幕')
        self.d.unlock()

    def open_url(self, url):
        """
        打开schema
        @param: url，schema链接，taobao://m.taobao.com/index.htm
        @return:
        """
        logger.info(f'打开url: {url}')
        self.d.open_url(url)

    @property
    def battery_info(self):
        """电池信息"""
        info = self.d.battery_info()
        logger.info(f'电池信息: {info}')
        return info

    @property
    def device_info(self):
        """设备信息"""
        info = self.d.device_info()
        logger.info(f'设备信息: {info}')
        return info

    @property
    def scale(self):
        """获取分辨率"""
        scale = self.d.scale
        logger.info(f'设备分辨率: {scale}')
        return scale









import qrunner
from qrunner import story, title


class HomePage(qrunner.Page):
    LOC_AD_CLOSE = {'id_': 'id/bottom_btn', 'desc': '首页广告关闭按钮'}
    LOC_MY = {'id_': 'id/bottom_view', 'index': 3, 'desc': '首页底部我的入口'}
    
    def go_my(self):
        self.elem(self.LOC_AD_CLOSE).click()
        self.elem(self.LOC_MY).click()


@story('首页')
class TestClass(qrunner.TestCase):
    
    def start(self):
        self.hp = HomePage(self.driver)
    
    @title('从首页进入我的页')
    def testcase(self):
        self.start_app()
        self.hp.go_my()
        self.assertText('我的订单')
        self.stop_app()


if __name__ == '__main__':
    qrunner.main(
        platform='android',
        device_id='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp'
    )



import qrunner
from qrunner import story, title


class HomePage(qrunner.Page):
    LOC_AD_CLOSE = {'label': 'close white big', 'desc': '首页广告关闭按钮'}
    LOC_MY = {'label': '我的', 'desc': '首页底部我的入口'}
    
    def go_my(self):
        self.elem(self.LOC_AD_CLOSE).click()
        self.elem(self.LOC_MY).click()


@story('首页')
class TestClass(qrunner.TestCase):

    def start(self):
        self.hp = HomePage(self.driver)

    @title('从首页进入我的页')
    def testcase(self):
        self.start_app()
        self.hp.go_my()
        self.assertText('我的订单')
        self.stop_app()


if __name__ == '__main__':
    qrunner.main(
        platform='ios',
        device_id='00008101-000E646A3C29003A',
        pkg_name='com.qizhidao.company'
    )



import os


def get_device_list():
    cmd = 'adb devices'
    output = os.popen(cmd).read()
    device_list = [item.split('\t')[0] for item in output.split('\n') if item.endswith('device')]
    return device_list


if __name__ == '__main__':
    print(get_device_list())




import time
from baseImage import Image, Rect
from qrunner.core.image.matching import SIFT
from qrunner.utils.log import logger


class ImageRecognition:
    def __init__(self, im_source_path, im_search_path):
        self.im_source_path = im_source_path
        self.im_search_path = im_search_path
        self.im_source = Image(im_source_path)
        self.im_search = Image(im_search_path)

    @staticmethod
    def mark(source_path, rect):
        """画框"""
        img = Image(source_path)
        img.rectangle(Rect(*rect), color=(0, 0, 255), thickness=3)
        img.imwrite(source_path)

    def match(self):
        """图片匹配，只获取最佳匹配"""
        logger.debug(f'开始匹配: 从 {self.im_source_path} 中 查找 {self.im_search_path}')
        match = SIFT()
        start = time.time()
        result = match.find_all_results(self.im_source, self.im_search)
        logger.debug(f'匹配耗时: {time.time() - start}')
        try:
            rect_tuple = result[0]['rect'].totuple()
        except:
            logger.debug(f'匹配失败, 结果如下: \n{result}')
            match_data = None
        else:
            top_left_x, top_left_y, width, height = rect_tuple
            logger.debug(f'匹配成功, 结果如下: \n{result}')
            match_data = {
                "rect": rect_tuple,
                "center": (top_left_x + width // 2, top_left_y + height // 2)
            }
        return match_data


class ImageElement:
    def __init__(self, driver, target_path=None):
        if target_path is None:
            raise AssertionError('请输入图片路径')
        self.target_path = target_path
        self.driver = driver

    def find_element(self, retry=3, timeout=3):
        center = None
        for _ in range(retry):
            logger.debug(f'开始查找元素: {self.target_path}')
            shot_path = self.driver.screenshot('screenshot.png')
            img_reg = ImageRecognition(shot_path, self.target_path)
            match_data = img_reg.match()
            if match_data is not None:
                logger.debug(f'元素: {self.target_path} 查找成功，结果如下: {match_data}')
                rect = match_data.get('rect')
                ImageRecognition.mark(shot_path, rect)
                center = match_data.get('center')
                break
            else:
                logger.debug(f'元素: {self.target_path} 查找失败，重试！！！')
                for i in range(timeout):
                    logger.debug(f'{i+1}s后重试')
                    time.sleep(1)
        return center

    def get_element(self, retry=3, timeout=3):
        center = self.find_element(retry=retry, timeout=timeout)
        if center is None:
            self.driver.screenshot(f'[控件 {self.target_path} 定位失败]')
            raise AssertionError(f'[控件 {self.target_path} 定位失败]')
        else:
            return center

    def click(self):
        """暂时只支持安卓和ios"""
        logger.debug(f'点击: {self.target_path}')
        x, y = self.get_element()
        self.driver.click(x, y)
        logger.debug('点击成功')






#! usr/bin/python
# -*- coding:utf-8 -*-
import cv2
import numpy as np
from baseImage import Image, Rect

from qrunner.core.image.matching import MatchTemplate
from qrunner.core.image.utils import (generate_result, get_keypoint_from_matches, keypoint_distance, rectangle_transform)
from qrunner.core.image.exceptions import (NoEnoughPointsError, PerspectiveTransformError, HomographyError, MatchResultError,
                                           InputImageError)
from typing import List


class BaseKeypoint(object):
    FILTER_RATIO = 1
    METHOD_NAME = None
    Dtype = None
    Place = None
    template = MatchTemplate()

    def __init__(self, threshold=0.8, rgb=True, **kwargs):
        """
        初始化

        Args:
            threshold: 识别阈值(0~1)
            rgb: 是否使用rgb通道进行校验
        """
        self.threshold = threshold
        self.rgb = rgb
        self.detector = self.create_detector(**kwargs)
        self.matcher = self.create_matcher(**kwargs)

    def create_matcher(self, **kwargs):
        raise NotImplementedError

    def create_detector(self, **kwargs):
        raise NotImplementedError

    def find_best_result(self, im_source, im_search, threshold=None, rgb=None, **kwargs):
        """
        通过特征点匹配,在im_source中找到最符合im_search的范围

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            threshold: 识别阈值(0~1)
            rgb: 是否使用rgb通道进行校验

        Returns:

        """
        max_count = 1
        ret = self.find_all_results(im_source=im_source, im_search=im_search, threshold=threshold, rgb=rgb,
                                    max_count=max_count, **kwargs)
        if ret:
            return ret[0]
        return None

    def find_all_results(self, im_source, im_search, threshold=None, rgb=None, max_count=10, max_iter_counts=20, distance_threshold=150):
        """
        通过特征点匹配,在im_source中找到全部符合im_search的范围

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            threshold: 识别阈值(0~1)
            rgb: 是否使用rgb通道进行校验
            max_count: 最多可以返回的匹配数量
            max_iter_counts: 最大的搜索次数,需要大于max_count
            distance_threshold: 距离阈值,特征点(first_point)大于该阈值后,不做后续筛选

        Returns:

        """
        threshold = self.threshold if threshold is None else threshold
        rgb = self.rgb if rgb is None else rgb

        im_source, im_search = self.input_image_check(im_source, im_search)
        result = []
        if im_source.channels == 1:
            rgb = False

        kp_src, des_src = self.get_keypoint_and_descriptor(image=im_source)
        kp_sch, des_sch = self.get_keypoint_and_descriptor(image=im_search)

        kp_src, kp_sch = list(kp_src), list(kp_sch)
        # 在特征点集中,匹配最接近的特征点
        matches = np.array(self.match_keypoint(des_sch=des_sch, des_src=des_src))
        kp_sch_point = np.array([(kp.pt[0], kp.pt[1], kp.angle) for kp in kp_sch])
        kp_src_matches_point = np.array([[(*kp_src[dMatch.trainIdx].pt, kp_src[dMatch.trainIdx].angle)
                                          if dMatch else np.nan for dMatch in match] for match in matches])
        _max_iter_counts = 0
        src_pop_list = []
        while True:
            # 这里没有用matches判断nan, 是因为类型不对
            if (np.count_nonzero(~np.isnan(kp_src_matches_point)) == 0) or (len(result) == max_count) or (_max_iter_counts >= max_iter_counts):
                break
            _max_iter_counts += 1
            filtered_good_point, angle, first_point = self.filter_good_point(matches=matches, kp_src=kp_src,
                                                                             kp_sch=kp_sch,
                                                                             kp_sch_point=kp_sch_point,
                                                                             kp_src_matches_point=kp_src_matches_point)
            if first_point.distance > distance_threshold:
                break

            rect, confidence = None, 0
            try:
                rect, confidence = self.extract_good_points(im_source=im_source, im_search=im_search, kp_src=kp_src,
                                                            kp_sch=kp_sch, good=filtered_good_point, angle=angle, rgb=rgb)
                # print(f'good:{len(filtered_good_point)}, rect={rect}, confidence={confidence}')
            except PerspectiveTransformError:
                pass
            finally:

                if rect and confidence >= threshold:
                    br, tl = rect.br, rect.tl
                    # 移除改范围内的所有特征点 ??有可能因为透视变换的原因，删除了多余的特征点
                    for index, match in enumerate(kp_src_matches_point):
                        x, y = match[:, 0], match[:, 1]
                        flag = np.argwhere((x < br.x) & (x > tl.x) & (y < br.y) & (y > tl.y))
                        for _index in flag:
                            src_pop_list.append(matches[index, _index][0].trainIdx)
                            kp_src_matches_point[index, _index, :] = np.nan
                            matches[index, _index] = np.nan
                    result.append(generate_result(rect, confidence))
                else:
                    for match in filtered_good_point:
                        flags = np.argwhere(matches[match.queryIdx, :] == match)
                        for _index in flags:
                            kp_src_matches_point[match.queryIdx, _index, :] = np.nan
                            matches[match.queryIdx, _index] = np.nan

        # 一下代码用于删除目标图片中的特征点,以后会用
        # src_pop_list = list(set(src_pop_list))
        # src_pop_list.sort(reverse=True)
        # mask = np.ones(len(des_src), dtype=bool)
        # for v in src_pop_list:
        #     kp_src.pop(v)
        #     mask[v] = False
        #
        # des_src = des_src[mask, ...]
        return result

    def get_keypoint_and_descriptor(self, image):
        """
        获取图像关键点(keypoint)与描述符(descriptor)

        Args:
            image: 待检测的灰度图像

        Returns:

        """
        if image.channels == 3:
            image = image.cvtColor(cv2.COLOR_BGR2GRAY).data
        else:
            image = image.data
        keypoint, descriptor = self.detector.detectAndCompute(image, None)

        if len(keypoint) < 2:
            raise NoEnoughPointsError('{} detect not enough feature points in input images'.format(self.METHOD_NAME))
        return keypoint, descriptor

    @staticmethod
    def filter_good_point(matches, kp_src, kp_sch, kp_sch_point, kp_src_matches_point):
        """ 筛选最佳点 """
        # 假设第一个点,及distance最小的点,为基准点
        sort_list = [sorted(match, key=lambda x: x is np.nan and float('inf') or x.distance)[0]
                     for match in matches]
        sort_list = [v for v in sort_list if v is not np.nan]

        first_good_point: cv2.DMatch = sorted(sort_list, key=lambda x: x.distance)[0]
        first_good_point_train: cv2.KeyPoint = kp_src[first_good_point.trainIdx]
        first_good_point_query: cv2.KeyPoint = kp_sch[first_good_point.queryIdx]
        first_good_point_angle = first_good_point_train.angle - first_good_point_query.angle

        def get_points_origin_angle(point_x, point_y, offset):
            points_origin_angle = np.arctan2(
                (point_y - offset.pt[1]),
                (point_x - offset.pt[0])
            ) * 180 / np.pi

            points_origin_angle = np.where(
                points_origin_angle == 0,
                points_origin_angle, points_origin_angle - offset.angle
            )
            points_origin_angle = np.where(
                points_origin_angle >= 0,
                points_origin_angle, points_origin_angle + 360
            )
            return points_origin_angle

        # 计算模板图像上,该点与其他特征点的旋转角
        first_good_point_sch_origin_angle = get_points_origin_angle(kp_sch_point[:, 0], kp_sch_point[:, 1],
                                                                    first_good_point_query)

        # 计算目标图像中,该点与其他特征点的夹角
        kp_sch_rotate_angle = kp_sch_point[:, 2] + first_good_point_angle
        kp_sch_rotate_angle = np.where(kp_sch_rotate_angle >= 360, kp_sch_rotate_angle - 360, kp_sch_rotate_angle)
        kp_sch_rotate_angle = kp_sch_rotate_angle.reshape(kp_sch_rotate_angle.shape + (1,))

        kp_src_angle = kp_src_matches_point[:, :, 2]
        good_point = np.array([matches[index][array[0]] for index, array in
                               enumerate(np.argsort(np.abs(kp_src_angle - kp_sch_rotate_angle)))])

        # 计算各点以first_good_point为原点的旋转角
        good_point_nan = (np.nan, np.nan)
        good_point_pt = np.array([good_point_nan if dMatch is np.nan else (*kp_src[dMatch.trainIdx].pt, )
                                 for dMatch in good_point])
        good_point_origin_angle = get_points_origin_angle(good_point_pt[:, 0], good_point_pt[:, 1],
                                                          first_good_point_train)
        threshold = round(5 / 360, 2) * 100
        point_bool = (np.abs(good_point_origin_angle - first_good_point_sch_origin_angle) / 360) * 100 < threshold
        _, index = np.unique(good_point_pt[point_bool], return_index=True, axis=0)
        good = good_point[point_bool]
        good = good[index]
        return good, int(first_good_point_angle), first_good_point

    def match_keypoint(self, des_sch, des_src, k=10):
        """
        特征点匹配

        Args:
            des_src: 待匹配图像的描述符集
            des_sch: 图片模板的描述符集
            k(int): 获取多少匹配点

        Returns:
            List[List[cv2.DMatch]]: 包含最匹配的描述符
        """
        # k=2表示每个特征点取出2个最匹配的对应点
        matches = self.matcher.knnMatch(des_sch, des_src, k)
        return matches

    def get_good_in_matches(self, matches):
        """
        特征点过滤

        Args:
            matches: 特征点集

        Returns:
            List[cv2.DMatch]: 过滤后的描述符集
        """
        if not matches:
            return None
        good = []
        for match_index in range(len(matches)):
            match = matches[match_index]
            for DMatch_index in range(len(match)):
                if match[DMatch_index].distance <= self.FILTER_RATIO * match[-1].distance:
                    good.append(match[DMatch_index])
        return good

    def extract_good_points(self, im_source, im_search, kp_src, kp_sch, good, angle, rgb):
        """
        根据匹配点(good)数量,提取识别区域

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            kp_src: 关键点集
            kp_sch: 关键点集
            good: 描述符集
            angle: 旋转角度
            rgb: 是否使用rgb通道进行校验

        Returns:
            范围,和置信度
        """
        len_good = len(good)
        confidence, rect, target_img = None, None, None

        if len_good == 0:
            pass
        elif len_good == 1:
            target_img, rect = self._handle_one_good_points(im_source=im_source, im_search=im_search,
                                                            kp_sch=kp_sch, kp_src=kp_src, good=good, angle=angle)
        elif len_good == 2:
            target_img, rect = self._handle_two_good_points(im_source=im_source, im_search=im_search,
                                                            kp_sch=kp_sch, kp_src=kp_src, good=good, angle=angle)
        elif len_good == 3:
            target_img, rect = self._handle_three_good_points(im_source=im_source, im_search=im_search,
                                                              kp_sch=kp_sch, kp_src=kp_src, good=good, angle=angle)
        else:  # len > 4
            target_img, rect = self._handle_many_good_points(im_source=im_source, im_search=im_search,
                                                             kp_sch=kp_sch, kp_src=kp_src, good=good)

        if target_img:
            confidence = self._cal_confidence(im_source=im_search, im_search=target_img, rgb=rgb)

        return rect, confidence

    def _handle_one_good_points(self, im_source, im_search, kp_src, kp_sch, good, angle):
        """
        特征点匹配数量等于1时,根据特征点的大小,对矩形进行缩放,并根据旋转角度,获取识别的目标图片

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            kp_sch: 关键点集
            kp_src: 关键点集
            good: 描述符集
            angle: 旋转角度

        Returns:
            待验证的图片
        """
        sch_point = get_keypoint_from_matches(kp=kp_sch, matches=good, mode='query')[0]
        src_point = get_keypoint_from_matches(kp=kp_src, matches=good, mode='train')[0]

        scale = src_point.size / sch_point.size
        h, w = im_search.size
        _h, _w = h * scale, w * scale
        src = np.float32(rectangle_transform(point=sch_point.pt, size=(h, w), mapping_point=src_point.pt,
                                             mapping_size=(_h, _w), angle=angle))
        dst = np.float32([[0, 0], [w, 0], [0, h], [w, h]])
        output = self._perspective_transform(im_source=im_source, im_search=im_search, src=src, dst=dst)
        rect = self._get_perspective_area_rect(im_source=im_source, src=src)
        return output, rect

    def _handle_two_good_points(self, im_source, im_search, kp_src, kp_sch, good, angle):
        """
        特征点匹配数量等于2时,根据两点距离差,对矩形进行缩放,并根据旋转角度,获取识别的目标图片

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            kp_sch: 关键点集
            kp_src: 关键点集
            good: 描述符集
            angle: 旋转角度

        Returns:
            待验证的图片
        """
        sch_point = get_keypoint_from_matches(kp=kp_sch, matches=good, mode='query')
        src_point = get_keypoint_from_matches(kp=kp_src, matches=good, mode='train')

        sch_distance = keypoint_distance(sch_point[0], sch_point[1])
        src_distance = keypoint_distance(src_point[0], src_point[1])

        try:
            scale = src_distance / sch_distance  # 计算缩放大小
        except ZeroDivisionError:
            if src_distance == sch_distance:
                scale = 1
            else:
                return None, None

        h, w = im_search.size
        _h, _w = h * scale, w * scale
        src = np.float32(rectangle_transform(point=sch_point[0].pt, size=(h, w), mapping_point=src_point[0].pt,
                                             mapping_size=(_h, _w), angle=angle))
        dst = np.float32([[0, 0], [w, 0], [0, h], [w, h]])
        output = self._perspective_transform(im_source=im_source, im_search=im_search, src=src, dst=dst)
        rect = self._get_perspective_area_rect(im_source=im_source, src=src)
        return output, rect

    def _handle_three_good_points(self, im_source, im_search, kp_src, kp_sch, good, angle):
        """
        特征点匹配数量等于3时,根据三个点组成的三角面积差,对矩形进行缩放,并根据旋转角度,获取识别的目标图片

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            kp_sch: 关键点集
            kp_src: 关键点集
            good: 描述符集
            angle: 旋转角度

        Returns:
            待验证的图片
        """
        sch_point = get_keypoint_from_matches(kp=kp_sch, matches=good, mode='query')
        src_point = get_keypoint_from_matches(kp=kp_src, matches=good, mode='train')

        def _area(point_list):
            p1_2 = keypoint_distance(point_list[0], point_list[1])
            p1_3 = keypoint_distance(point_list[0], point_list[2])
            p2_3 = keypoint_distance(point_list[1], point_list[2])

            s = (p1_2 + p1_3 + p2_3) / 2
            area = (s * (s - p1_2) * (s - p1_3) * (s - p2_3)) ** 0.5
            return area

        sch_area = _area(sch_point)
        src_area = _area(src_point)

        try:
            scale = src_area / sch_area  # 计算缩放大小
        except ZeroDivisionError:
            if sch_area == src_area:
                scale = 1
            else:
                return None, None

        h, w = im_search.size
        _h, _w = h * scale, w * scale
        src = np.float32(rectangle_transform(point=sch_point[0].pt, size=(h, w), mapping_point=src_point[0].pt,
                                             mapping_size=(_h, _w), angle=angle))
        dst = np.float32([[0, 0], [w, 0], [0, h], [w, h]])
        output = self._perspective_transform(im_source=im_source, im_search=im_search, src=src, dst=dst)
        rect = self._get_perspective_area_rect(im_source=im_source, src=src)
        return output, rect

    def _handle_many_good_points(self, im_source, im_search, kp_src, kp_sch, good):
        """
        特征点匹配数量>=4时,使用单矩阵映射,获取识别的目标图片

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            kp_sch: 关键点集
            kp_src: 关键点集
            good: 描述符集

        Returns:
            透视变换后的图片
        """

        sch_pts, img_pts = np.float32([kp_sch[m.queryIdx].pt for m in good]).reshape(
            -1, 1, 2), np.float32([kp_src[m.trainIdx].pt for m in good]).reshape(-1, 1, 2)
        # M是转化矩阵
        M, mask = self._find_homography(sch_pts, img_pts)
        # 计算四个角矩阵变换后的坐标，也就是在大图中的目标区域的顶点坐标:
        h, w = im_search.size
        h_s, w_s = im_source.size
        pts = np.float32([[0, 0], [0, h - 1], [w - 1, h - 1], [w - 1, 0]]).reshape(-1, 1, 2)
        try:
            dst: np.ndarray = cv2.perspectiveTransform(pts, M)
            # img = im_source.clone().data
            # img2 = cv2.polylines(img, [np.int32(dst)], True, 255, 3, cv2.LINE_AA)
            # Image(img).imshow('dst')
            pypts = [tuple(npt[0]) for npt in dst.tolist()]
            src = np.array([pypts[0], pypts[3], pypts[1], pypts[2]], dtype=np.float32)
            dst = np.float32([[0, 0], [w, 0], [0, h], [w, h]])
            output = self._perspective_transform(im_source=im_source, im_search=im_search, src=src, dst=dst)
        except cv2.error as err:
            raise PerspectiveTransformError(err)

        # img = im_source.clone().data
        # cv2.polylines(img, [np.int32(dst)], True, 255, 3, cv2.LINE_AA)
        # Image(img).imshow()
        # cv2.waitKey(0)

        rect = self._get_perspective_area_rect(im_source=im_source, src=src)
        return output, rect

    @staticmethod
    def _target_image_crop(img, rect):
        """
        截取目标图像

        Args:
            img: 图像
            rect: 图像范围

        Returns:
            裁剪后的图像
        """
        try:
            target_img = img.crop(rect)
        except OverflowError:
            raise MatchResultError(f"Target area({rect}) out of screen{img.size}")
        return target_img

    def _cal_confidence(self, im_source, im_search, rgb):
        """
        将截图和识别结果缩放到大小一致,并计算可信度

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            rgb:是否使用rgb通道进行校验

        Returns:

        """
        h, w = im_source.size
        im_search = im_search.resize(w, h)
        if rgb:
            confidence = self.template.cal_rgb_confidence(im_source=im_source, im_search=im_search)
        else:
            confidence = self.template.cal_ccoeff_confidence(im_source=im_source, im_search=im_search)

        confidence = (1 + confidence) / 2
        return confidence

    def input_image_check(self, im_source, im_search):
        im_source = self._image_check(im_source)
        im_search = self._image_check(im_search)

        if im_source.place != im_search.place:
            raise InputImageError('输入图片类型必须相同, source={}, search={}'.format(im_source.place, im_search.place))
        elif im_source.dtype != im_search.dtype:
            raise InputImageError('输入图片数据类型必须相同, source={}, search={}'.format(im_source.dtype, im_search.dtype))
        elif im_source.channels != im_search.channels:
            raise InputImageError('输入图片通道必须相同, source={}, search={}'.format(im_source.channels, im_search.channels))

        return im_source, im_search

    def _image_check(self, data):
        if not isinstance(data, Image):
            data = Image(data, dtype=self.Dtype)

        if data.place not in self.Place:
            raise TypeError(f'{self.METHOD_NAME}方法,Image类型必须为(Place.UMat, Place.Ndarray)')
        return data

    @staticmethod
    def _find_homography(sch_pts, src_pts):
        """
        多组特征点对时，求取单向性矩阵
        """
        try:
            # M, mask = cv2.findHomography(sch_pts, src_pts, cv2.RANSAC)
            M, mask = cv2.findHomography(sch_pts, src_pts, cv2.USAC_MAGSAC, 4.0, None, 2000, 0.99)
        except cv2.error:
            import traceback
            traceback.print_exc()
            raise HomographyError("OpenCV error in _find_homography()...")
        else:
            if mask is None:
                raise HomographyError("In _find_homography(), find no mask...")
            else:
                return M, mask

    @staticmethod
    def _perspective_transform(im_source, im_search, src, dst):
        """
        根据四对对应点计算透视变换, 并裁剪相应图片

        Args:
            im_source: 待匹配图像
            im_search: 待匹配模板
            src: 目标图像中相应四边形顶点的坐标 (左上,右上,左下,右下)
            dst: 源图像中四边形顶点的坐标 (左上,右上,左下,右下)

        Returns:

        """
        h, w = im_search.size
        matrix = cv2.getPerspectiveTransform(src=src, dst=dst)
        # warpPerspective https://github.com/opencv/opencv/issues/11784
        output = im_source.warpPerspective(matrix, size=(w, h), flags=cv2.INTER_CUBIC)

        return output

    @staticmethod
    def _get_perspective_area_rect(im_source, src):
        """
        根据矩形四个顶点坐标,获取在原图中的最大外接矩形

        Args:
            im_source: 待匹配图像
            src: 目标图像中相应四边形顶点的坐标

        Returns:
            最大外接矩形
        """
        h, w = im_source.size

        x = [int(i[0]) for i in src]
        y = [int(i[1]) for i in src]
        x_min, x_max = min(x), max(x)
        y_min, y_max = min(y), max(y)
        # 挑选出目标矩形区域可能会有越界情况，越界时直接将其置为边界：
        # 超出左边界取0，超出右边界取w_s-1，超出下边界取0，超出上边界取h_s-1
        # 当x_min小于0时，取0。  x_max小于0时，取0。
        x_min, x_max = int(max(x_min, 0)), int(max(x_max, 0))
        # 当x_min大于w_s时，取值w_s-1。  x_max大于w_s-1时，取w_s-1。
        x_min, x_max = int(min(x_min, w - 1)), int(min(x_max, w - 1))
        # 当y_min小于0时，取0。  y_max小于0时，取0。
        y_min, y_max = int(max(y_min, 0)), int(max(y_max, 0))
        # 当y_min大于h_s时，取值h_s-1。  y_max大于h_s-1时，取h_s-1。
        y_min, y_max = int(min(y_min, h - 1)), int(min(y_max, h - 1))
        rect = Rect(x=x_min, y=y_min, width=(x_max - x_min), height=(y_max - y_min))
        return rect



import json
import os.path


# 从allure数据中获取用例执行情况
def get_allure_data(allure_path):
    # 从json文件中获取执行结果列表
    data_list = []
    for filename in os.listdir(allure_path):
        if filename.endswith('result.json'):
            # 读result.json
            print(os.path.join(allure_path, filename))
            with open(os.path.join(allure_path, filename), 'r', encoding='UTF-8') as f:
                content = json.load(f)

            # 获取需要的字段
            full_name = content.get('fullName')
            status = content.get('status')
            data = {'full_name': full_name, 'status': status}

            # 去重，失败重试后会有重复执行结果（这里可能会有一个bug，重试成功后会同时显示失败和成功的用例）
            for item in data_list:
                if full_name == item.get('full_name'):
                    break
            else:
                data_list.append(data)

    # 根据执行结果进行数据统计
    print(data_list)
    total_num = len(data_list)
    pass_list = [data for data in data_list if data.get('status') == 'passed']
    pass_num = len(pass_list)
    fail_num = total_num - pass_num
    rate = round((pass_num / total_num) * 100, 2)
    data = {
        'total': total_num,
        'passed': pass_num,
        'fail': fail_num,
        'rate': rate
    }
    return data


if __name__ == '__main__':
    print(get_allure_data('/Users/UI/PycharmProjects/qrunner_new_gitee/allure-results'))






#! usr/bin/python
# -*- coding:utf-8 -*-
import math
import cv2
import numpy as np
from typing import Union


def generate_result(rect, confi):
    """Format the result: 定义图像识别结果格式."""
    ret = {
        'rect': rect,
        'confidence': confi,
    }
    return ret


def keypoint_distance(kp1, kp2):
    """求两个keypoint的两点之间距离"""
    if isinstance(kp1, cv2.KeyPoint):
        kp1 = kp1.pt
    elif isinstance(kp1, (list, tuple)):
        kp1 = kp1
    else:
        raise ValueError('kp1需要时keypoint或直接是坐标, kp1={}'.format(kp1))

    if isinstance(kp2, cv2.KeyPoint):
        kp2 = kp2.pt
    elif isinstance(kp2, (list, tuple)):
        kp2 = kp2
    else:
        raise ValueError('kp2需要时keypoint或直接是坐标, kp1={}'.format(kp2))

    x = kp1[0] - kp2[0]
    y = kp1[1] - kp2[1]
    return math.sqrt((x ** 2) + (y ** 2))


def keypoint_angle(kp1, kp2):
    """求两个keypoint的夹角 """
    k = [
        (kp1.angle - 180) if kp1.angle >= 180 else kp1.angle,
        (kp2.angle - 180) if kp2.angle >= 180 else kp2.angle
    ]
    if k[0] == k[1]:
        return 0
    else:
        return abs(k[0] - k[1])


def get_keypoint_from_matches(kp, matches, mode):
    res = []
    if mode == 'query':
        for match in matches:
            res.append(kp[match.queryIdx])
    elif mode == 'train':
        for match in matches:
            res.append(kp[match.trainIdx])

    return res


def keypoint_origin_angle(kp1, kp2):
    """
    以kp1为原点,计算kp2的旋转角度
    """
    origin_point = kp1.pt
    train_point = kp2.pt

    point = (abs(origin_point[0] - train_point[0]), abs(origin_point[1] - train_point[1]))

    x_quadrant = (1, 4)
    y_quadrant = (3, 4)
    if origin_point[0] > train_point[0]:
        x_quadrant = (2, 3)

    if origin_point[1] > train_point[1]:
        y_quadrant = (1, 2)
    point_quadrant = list(set(x_quadrant).intersection(set(y_quadrant)))[0]

    x, y = point[::-1]
    angle = math.degrees(math.atan2(x, y))
    if point_quadrant == 4:
        angle = angle
    elif point_quadrant == 3:
        angle = 180 - angle
    elif point_quadrant == 2:
        angle = 180 + angle
    elif point_quadrant == 1:
        angle = 360 - angle

    return angle


def _mapping_angle_distance(distance, origin_angle, angle):
    """

    Args:
        distance: 距离
        origin_angle: 对应原点的角度
        angle: 旋转角度

    """
    _angle = origin_angle + angle
    _y = distance * math.cos((math.pi * _angle) / 180)
    _x = distance * math.sin((math.pi * _angle) / 180)
    return round(_x, 3), round(_y, 3)


def rectangle_transform(point, size, mapping_point, mapping_size, angle):
    """
    根据point,找出mapping_point映射的矩形顶点坐标

    Args:
        point: 坐标在矩形中的坐标
        size: 矩形的大小(h, w)
        mapping_point: 映射矩形的坐标
        mapping_size: 映射矩形的大小(h, w)
        angle: 旋转角度

    Returns:

    """
    h, w = size[0], size[1]
    _h, _w = mapping_size[0], mapping_size[1]

    h_scale = _h / h
    w_scale = _w / w

    tl = keypoint_distance((0, 0), point)  # 左上
    tr = keypoint_distance((w, 0), point)  # 右上
    bl = keypoint_distance((0, h), point)  # 左下
    br = keypoint_distance((w, h), point)  # 右下

    # x = np.float32([point[1], point[1], (h - point[1]), (h - point[1])])
    # y = np.float32([point[0], (w - point[0]), point[0], (w - point[0])])
    # A, B, C, D = cv2.phase(x, y, angleInDegrees=True)
    A = math.degrees(math.atan2(point[0], point[1]))
    B = math.degrees(math.atan2((w - point[0]), point[1]))
    C = math.degrees(math.atan2(point[0], (h - point[1])))
    D = math.degrees(math.atan2((w - point[0]), (h - point[1])))

    new_tl = _mapping_angle_distance(tl, A, angle=angle)
    new_tl = (-new_tl[0] * w_scale, -new_tl[1] * h_scale)
    new_tl = (mapping_point[0] + new_tl[0], mapping_point[1] + new_tl[1])

    new_tr = _mapping_angle_distance(tr, B, angle=angle)
    new_tr = (new_tr[0] * w_scale, -new_tr[1] * h_scale)
    new_tr = (mapping_point[0] + new_tr[0], mapping_point[1] + new_tr[1])

    new_bl = _mapping_angle_distance(bl, C, angle=angle)
    new_bl = (-new_bl[0] * w_scale, new_bl[1] * h_scale)
    new_bl = (mapping_point[0] + new_bl[0], mapping_point[1] + new_bl[1])

    new_br = _mapping_angle_distance(br, D, angle=angle)
    new_br = (new_br[0] * w_scale, new_br[1] * h_scale)
    new_br = (mapping_point[0] + new_br[0], mapping_point[1] + new_br[1])

    return [new_tl, new_tr, new_bl, new_br]



#! usr/bin/python
# -*- coding:utf-8 -*-
""" opencv matchTemplate"""
import warnings
import cv2
import numpy as np
from baseImage import Image, Rect
from baseImage.constant import Place

from qrunner.core.image.exceptions import MatchResultError, NoModuleError, InputImageError
from qrunner.core.image.utils import generate_result
from typing import Union


class MatchTemplate(object):
    METHOD_NAME = 'tpl'
    Dtype = np.uint8
    Place = (Place.Ndarray, )

    def __init__(self, threshold=0.8, rgb=True):
        """
        初始化

        Args:
            threshold: 识别阈值(0~1)
            rgb: 是否使用rgb通道进行校验
        """
        assert 0 <= threshold <= 1, 'threshold 取值在0到1直接'

        self.threshold = threshold
        self.rgb = rgb
        self.matcher = cv2.matchTemplate

    def find_best_result(self, im_source, im_search, threshold=None, rgb=None):
        """
        模板匹配, 返回匹配度最高且大于阈值的范围

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            threshold: 识别阈值(0~1)
            rgb: 是否使用rgb通道进行校验

        Returns:
            generate_result
        """
        threshold = threshold or self.threshold
        rgb = rgb or self.rgb

        im_source, im_search = self.input_image_check(im_source, im_search)
        if im_source.channels == 1:
            rgb = False

        result = self._get_template_result_matrix(im_source=im_source, im_search=im_search)
        # 找到最佳匹配项

        min_val, max_val, min_loc, max_loc = self.minMaxLoc(result.data)
        h, w = im_search.size
        # 求可信度
        crop_rect = Rect(max_loc[0], max_loc[1], w, h)

        confidence = self.cal_confidence(im_source, im_search, crop_rect, max_val, rgb)
        # 如果可信度小于threshold,则返回None
        if confidence < (threshold or self.threshold):
            return None
        x, y = max_loc
        rect = Rect(x=x, y=y, width=w, height=h)
        return generate_result(rect, confidence)

    def find_all_results(self, im_source, im_search, threshold=None, rgb=None, max_count=10):
        """
        模板匹配, 返回匹配度大于阈值的范围, 且最大数量不超过max_count

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            threshold:: 识别阈值(0~1)
            rgb: 是否使用rgb通道进行校验
            max_count: 最多匹配数量

        Returns:

        """
        threshold = threshold or self.threshold
        rgb = rgb or self.rgb

        im_source, im_search = self.input_image_check(im_source, im_search)
        if im_source.channels == 1:
            rgb = False

        result = self._get_template_result_matrix(im_source=im_source, im_search=im_search)
        results = []
        # 找到最佳匹配项
        h, w = im_search.size
        while True:
            min_val, max_val, min_loc, max_loc = self.minMaxLoc(result.data)
            img_crop = im_source.crop(Rect(max_loc[0], max_loc[1], w, h))
            confidence = self._get_confidence_from_matrix(img_crop, im_search, max_val=max_val, rgb=rgb)
            x, y = max_loc
            rect = Rect(x, y, w, h)

            if (confidence < (threshold or self.threshold)) or len(results) >= max_count:
                break
            results.append(generate_result(rect, confidence))
            result.rectangle(rect=Rect(int(max_loc[0] - w / 2), int(max_loc[1] - h / 2), w, h), color=(0, 0, 0), thickness=-1)

        return results if results else None

    def _get_template_result_matrix(self, im_source, im_search):
        """求取模板匹配的结果矩阵."""
        if im_source.channels == 3:
            i_gray = im_source.cvtColor(cv2.COLOR_BGR2GRAY).data
            s_gray = im_search.cvtColor(cv2.COLOR_BGR2GRAY).data
        else:
            i_gray = im_source.data
            s_gray = im_search.data

        result = self.match(i_gray, s_gray)
        result = Image(data=result, dtype=np.float32, clone=False, place=3)
        return result

    def input_image_check(self, im_source, im_search):
        im_source = self._image_check(im_source)
        im_search = self._image_check(im_search)

        if im_source.place != im_search.place:
            raise InputImageError('输入图片类型必须相同, source={}, search={}'.format(im_source.place, im_search.place))
        elif im_source.dtype != im_search.dtype:
            raise InputImageError('输入图片数据类型必须相同, source={}, search={}'.format(im_source.dtype, im_search.dtype))
        elif im_source.channels != im_search.channels:
            raise InputImageError('输入图片通道必须相同, source={}, search={}'.format(im_source.channels, im_search.channels))

        if im_source.place == Place.UMat:
            warnings.warn('Umat has error,will clone new image with np.ndarray '
                          '(https://github.com/opencv/opencv/issues/21788)')
            im_source = Image(im_source, place=Place.Ndarray, dtype=im_source.dtype)
            im_search = Image(im_search, place=Place.Ndarray, dtype=im_search.dtype)

        return im_source, im_search

    def _image_check(self, data):
        if not isinstance(data, Image):
            data = Image(data, dtype=self.Dtype)

        if data.place not in self.Place:
            raise TypeError('Image类型必须为(Place.UMat, Place.Ndarray)')
        return data

    @staticmethod
    def minMaxLoc(result):
        return cv2.minMaxLoc(result)

    def match(self, img1, img2):
        return self.matcher(img1, img2, cv2.TM_CCOEFF_NORMED)

    def cal_confidence(self, im_source, im_search, crop_rect, max_val, rgb):
        """
        将截图和识别结果缩放到大小一致,并计算可信度

        Args:
            im_source: 待匹配图像
            im_search: 图片模板
            crop_rect: 需要在im_source截取的区域
            max_val: matchTemplate得到的最大值
            rgb: 是否使用rgb通道进行校验

        Returns:
            float: 可信度(0~1)
        """
        try:
            target_img = im_source.crop(crop_rect)
        except OverflowError:
            raise MatchResultError(f"Target area({crop_rect}) out of screen{im_source.size}")
        confidence = self._get_confidence_from_matrix(target_img, im_search, max_val, rgb)
        return confidence

    def cal_rgb_confidence(self, im_source, im_search):
        """
        计算两张图片图片rgb三通道的置信度

        Args:
            im_source: 待匹配图像
            im_search: 图片模板

        Returns:
            float: 最小置信度
        """
        im_search = im_search.copyMakeBorder(10, 10, 10, 10, cv2.BORDER_REPLICATE)

        img_src_hsv = im_source.cvtColor(cv2.COLOR_BGR2HSV)
        img_sch_hsv = im_search.cvtColor(cv2.COLOR_BGR2HSV)

        src_split = img_src_hsv.split()
        sch_split = img_sch_hsv.split()

        # 计算BGR三通道的confidence，存入bgr_confidence:
        bgr_confidence = [0, 0, 0]
        for i in range(3):
            res_temp = self.match(sch_split[i], src_split[i])
            min_val, max_val, min_loc, max_loc = self.minMaxLoc(res_temp)
            bgr_confidence[i] = max_val

        return min(bgr_confidence)

    def cal_ccoeff_confidence(self, im_source, im_search):
        if im_source.channels == 3:
            img_src_gray = im_source.cvtColor(cv2.COLOR_BGR2GRAY).data
        else:
            img_src_gray = im_source.data

        if im_search.channels == 3:
            img_sch_gray = im_search.cvtColor(cv2.COLOR_BGR2GRAY).data
        else:
            img_sch_gray = im_search.data

        res_temp = self.match(img_sch_gray, img_src_gray)
        min_val, max_val, min_loc, max_loc = self.minMaxLoc(res_temp)
        return max_val

    def _get_confidence_from_matrix(self, img_crop, im_search, max_val, rgb):
        """根据结果矩阵求出confidence."""
        # 求取可信度:
        if rgb:
            # 如果有颜色校验,对目标区域进行BGR三通道校验:
            confidence = self.cal_rgb_confidence(img_crop, im_search)
        else:
            confidence = max_val
        return confidence


class CudaMatchTemplate(MatchTemplate):
    METHOD_NAME = 'cuda_tpl'
    Dtype = np.uint8
    Place = (Place.GpuMat, )

    def __init__(self, threshold=0.8, rgb=True):
        super(CudaMatchTemplate, self).__init__(threshold=threshold, rgb=rgb)
        try:
            self.matcher = cv2.cuda.createTemplateMatching(cv2.CV_8U, cv2.TM_CCOEFF_NORMED)
        except AttributeError:
            raise NoModuleError('create CUDA TemplateMatching Error')

    @staticmethod
    def minMaxLoc(result):
        return cv2.cuda.minMaxLoc(result)

    def match(self, img1, img2):
        return self.matcher.match(img1, img2)



import argparse
from qrunner import __version__, __description__, logger
from qrunner.scaffold import create_scaffold
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import IEDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager
from qrunner.utils.webdriver_manager_extend import ChromeDriverManager


def main():
    """ API test: parse command line options and run commands.
    """
    parser = argparse.ArgumentParser(description=__description__)
    parser.add_argument("-v", "--version", dest="version", action='store_true', help="版本号")
    parser.add_argument("-n", "--project_name", dest="project_name", help="项目名称")
    parser.add_argument("-p", "--platform", dest="platform", help="所属平台")
    parser.add_argument("-i", "--install", dest="install", help="浏览器驱动名称")

    args = parser.parse_args()
    version = args.version
    project = args.project_name
    platform = args.platform
    install = args.install

    if version:
        print(__version__)
        return 0
    if project:
        if platform:
            create_scaffold(project, platform)
        else:
            print('请输入-p参数设置所属平台')
        return 0
    if install:
        install_driver(install)
        return 0


def install_driver(browser: str) -> None:
    """
    Download and install the browser driver
    :param browser: The Driver to download. Pass as `chrome/firefox/ie/edge`. Default Chrome.
    :return:
    """

    if browser == "chrome":
        driver_path = ChromeDriverManager().install()
        logger.info(f"Chrome Driver[==>] {driver_path}")
    elif browser == "firefox":
        driver_path = GeckoDriverManager().install()
        logger.info(f"Firefox Driver[==>] {driver_path}")
    elif browser == "ie":
        driver_path = IEDriverManager().install()
        logger.info(f"IE Driver[==>] {driver_path}")
    elif browser == "edge":
        driver_path = EdgeChromiumDriverManager().install()
        logger.info(f"Edge Driver[==>] {driver_path}")
    else:
        raise NameError(f"Not found '{browser}' browser driver.")




import logging
import os
import sys
import time
import allure
from selenium import webdriver
from selenium.webdriver import DesiredCapabilities
from selenium.webdriver import ChromeOptions
from selenium.webdriver import FirefoxOptions
from selenium.webdriver import EdgeOptions
from qrunner.utils.webdriver_manager_extend import ChromeDriverManager
from webdriver_manager.firefox import GeckoDriverManager
from webdriver_manager.microsoft import EdgeChromiumDriverManager, IEDriverManager
from qrunner.utils.log import logger
from qrunner.utils.config import config
from urllib import parse
from selenium.webdriver.remote.remote_connection import LOGGER
from qrunner.utils.exceptions import ScreenFailException
from selenium.webdriver.firefox.service import Service as fService
from selenium.webdriver.ie.service import Service as iService
from selenium.webdriver.edge.service import Service as eService
from selenium.webdriver.chrome.service import Service as cService

LOGGER.setLevel(logging.WARNING)


class ChromeConfig:
    headless = False
    options = None
    command_executor = ""


class FirefoxConfig:
    headless = False
    options = None
    command_executor = ""


class IEConfig:
    command_executor = ""


class EdgeConfig:
    headless = False
    options = None
    command_executor = ""


class SafariConfig:
    executable_path = "/usr/bin/safaridriver"
    command_executor = ""


class Browser(object):
    """
    根据关键词初始化浏览器操作句柄，
    如'chrome、google chrome、gc'代表chrome浏览器，
    如'firefox、ff'代表火狐浏览器，
    如'internet explorer、ie、IE'代表IE浏览器，
    如'edge'代表edge浏览器，
    如'safari'代表safari浏览器
    """
    name = None

    def __new__(cls, name=None):
        cls.name = name

        if (cls.name is None) or (cls.name in ["chrome", "google chrome", "gc"]):
            return cls.chrome()
        elif cls.name in ['internet explorer', 'ie', 'IE']:
            return cls.ie()
        elif cls.name in ['firefox', 'ff']:
            return cls.firefox()
        elif cls.name == 'edge':
            return cls.edge()
        elif cls.name == 'safari':
            return cls.safari()
        raise NameError(f"Not found {cls.name} browser")

    @staticmethod
    def chrome():
        logger.debug(ChromeConfig.headless)
        if ChromeConfig.options is None:
            chrome_options = ChromeOptions()
        else:
            chrome_options = ChromeConfig.options

        if ChromeConfig.headless is True:
            chrome_options.add_argument('--headless')
            chrome_options.add_argument(
                f'user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.79 Safari/537.36')

        is_grid = False
        if ChromeConfig.command_executor == "":
            driver = webdriver.Chrome(options=chrome_options,
                                      service=cService(ChromeDriverManager().install()))
        elif ChromeConfig.command_executor[:4] == "http":
            is_grid = True
            driver = webdriver.Remote(options=chrome_options,
                                      command_executor=ChromeConfig.command_executor,
                                      desired_capabilities=DesiredCapabilities.CHROME.copy())
        else:
            driver = webdriver.Chrome(options=chrome_options,
                                      executable_path=ChromeConfig.command_executor)

        if is_grid is False:
            driver.execute_cdp_cmd("Page.addScriptToEvaluateOnNewDocument", {
                "source": """
                            Object.defineProperty(navigator, 'webdriver', {
                            get: () => undefined
                            })"""
            })

        return driver

    @staticmethod
    def firefox():
        if FirefoxConfig.options is None:
            firefox_options = FirefoxOptions()
        else:
            firefox_options = FirefoxConfig.options

        if FirefoxConfig.headless is True:
            firefox_options.add_argument('-headless')
            firefox_options.add_argument(
                f'user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/79.0.3945.79 Safari/537.36')

        if FirefoxConfig.command_executor == "":
            driver = webdriver.Firefox(options=firefox_options,
                                       service=fService(GeckoDriverManager().install()))
        elif FirefoxConfig.command_executor[:4] == "http":
            driver = webdriver.Remote(options=firefox_options,
                                      command_executor=FirefoxConfig.command_executor,
                                      desired_capabilities=DesiredCapabilities.FIREFOX.copy())
        else:
            driver = webdriver.Firefox(options=firefox_options,
                                       executable_path=FirefoxConfig.command_executor)

        return driver

    @staticmethod
    def ie():
        if IEConfig.command_executor == "":
            driver = webdriver.Ie(service=iService(IEDriverManager().install()))
        elif IEConfig.command_executor[:4] == "http":
            driver = webdriver.Remote(command_executor=IEConfig.command_executor,
                                      desired_capabilities=DesiredCapabilities.INTERNETEXPLORER.copy())
        else:
            driver = webdriver.Ie(executable_path=IEConfig.command_executor)

        return driver

    @staticmethod
    def edge():
        if EdgeConfig.options is None:
            edge_options = EdgeOptions()
        else:
            edge_options = EdgeConfig.options

        if EdgeConfig.headless is True:
            edge_options.headless = True

        if EdgeConfig.command_executor == "":
            driver = webdriver.Edge(options=edge_options,
                                    service=eService(EdgeChromiumDriverManager().install()))
        elif EdgeConfig.command_executor[:4] == "http":
            driver = webdriver.Remote(options=edge_options,
                                      command_executor=EdgeConfig.command_executor,
                                      desired_capabilities=DesiredCapabilities.EDGE.copy())
        else:
            driver = webdriver.Edge(options=edge_options,
                                    executable_path=EdgeConfig.command_executor)

        return driver

    @staticmethod
    def safari():
        if SafariConfig.command_executor != "" and SafariConfig.command_executor[:4] == "http":
            return webdriver.Remote(command_executor=SafariConfig.command_executor,
                                    desired_capabilities=DesiredCapabilities.SAFARI.copy())
        return webdriver.Safari(executable_path=SafariConfig.executable_path)


class WebDriver(object):
    _instance = {}

    def __new__(cls, browser_name=None):
        if browser_name not in cls._instance:
            cls._instance[browser_name] = super().__new__(cls)
        return cls._instance[browser_name]

    def __init__(self, browser_name=None):
        self.d = Browser(browser_name)
        self.d.set_page_load_timeout(config.get_timeout())
        if ChromeConfig.headless:
            self.d.set_window_size(1920, 1080)
        else:
            self.d.maximize_window()

    @classmethod
    def get_instance(cls, browser_name=None):
        """Create singleton"""
        if browser_name not in cls._instance:
            logger.info(f'[{browser_name}] Create web driver singleton')
            return WebDriver(browser_name)
        return WebDriver._instance[browser_name]

    def open_url(self, url=None, login=True):
        if url is None:
            base_url = config.get_host()
            if not base_url:
                logger.debug('请设置base_url')
                sys.exit()
            url = base_url
        else:
            if 'http' not in url:
                base_url = config.get_host()
                if not base_url:
                    logger.debug('请设置base_url')
                    sys.exit()
                url = parse.urljoin(base_url, url)
        logger.info(f'访问: {url}')
        self.d.get(url)
        # 把默认请求头添加到cookie中
        if login:
            headers = config.get_login()
            logger.debug(headers)
            cookies = []
            if headers:
                for k, v in headers.items():
                    cookies.append({"name": k, "value": v})
                self.add_cookies(cookies)
                self.refresh()

    def back(self):
        logger.info('返回上一页')
        self.d.back()

    def screenshot(self, file_name):
        """
        截图并保存到预定路径
        @param file_name: foo.png or fool
        @return:
        """
        try:
            # 把文件名处理成test.png的样式
            if '.' in file_name:
                file_name = file_name.split(r'.')[0]
            # 截图并保存到当前目录的images文件夹中
            img_dir = os.path.join(os.getcwd(), 'images')
            if os.path.exists(img_dir) is False:
                os.mkdir(img_dir)
            time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
            file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
            self.d.save_screenshot(file_path)
            # 上传allure报告
            allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
            return file_path
        except Exception as e:
            raise ScreenFailException(f'{file_name} 截图失败\n{str(e)}')

    # def screenshot_and_mark(self, file_name, rect):
    #     """给图片指定范围画上红框
    #     rect: [x, y, width, height]
    #     x: 左上坐标x
    #     y：左上角坐标y
    #     width：矩形宽度
    #     height：矩形高度
    #     """
    #     # 把文件名处理成test.png的样式
    #     if '.' in file_name:
    #         file_name = file_name.split(r'.')[0]
    #     # 截图并保存到当前目录的images文件夹中
    #     img_dir = os.path.join(os.getcwd(), 'images')
    #     if os.path.exists(img_dir) is False:
    #         os.mkdir(img_dir)
    #     time_str = time.strftime('%Y年%m月%d日 %H时%M分%S秒')
    #     file_path = os.path.join(img_dir, f'{time_str}-{file_name}.png')
    #     self.d.save_screenshot(file_path)
    #     # 画框
    #     ImageRecognition.mark(file_path, rect)
    #     # 上传allure报告
    #     allure.attach.file(file_path, attachment_type=allure.attachment_type.PNG, name=f'{file_name}.png')
    #     return file_path

    def get_page_content(self):
        page_source = self.d.page_source
        logger.info(f'获取页面内容: \n{page_source}')
        return page_source

    # @property
    # def rect(self):
    #     logger.info('获取窗口的坐标及宽高')
    #     return self.d.get_window_position()

    def max_window(self):
        logger.info('窗口设置全屏')
        self.d.maximize_window()

    def set_window(self, width, height):
        logger.info('设置窗口长宽')
        self.d.set_window_size(width, height)

    def get_windows(self):
        logger.info(f'获取当前打开的窗口列表')
        return self.d.window_handles

    def switch_window(self, old_windows):
        logger.info('切换到最新的window')
        current_windows = self.d.window_handles
        newest_window = [window for window in current_windows if window not in old_windows][0]
        self.d.switch_to.window(newest_window)

    def window_scroll(self, width, height):
        logger.info(f'设置scroll bar的宽高')
        js = "window.scrollTo({w},{h});".format(w=str(width), h=str(height))
        self.d.execute_script(js)

    def switch_to_frame(self, frame_id):
        logger.info(f'切换到frame {frame_id}')
        self.d.switch_to.frame(frame_id)

    def switch_to_frame_out(self):
        logger.info('从frame中切出来')
        self.d.switch_to.default_content()

    def execute_js(self, script, *args):
        logger.info(f'执行js脚本: \n{script}')
        self.d.execute_script(script, *args)

    def click(self, element):
        logger.info(f'点击元素: {element}')
        self.d.execute_script('arguments[0].click();', element)

    def quit(self):
        logger.info('退出浏览器')
        self.d.quit()

    def close(self):
        logger.info('关闭当前页签')
        self.d.close()

    def add_cookies(self, cookies: list):
        for cookie in cookies:
            self.d.add_cookie(cookie)

    def get_cookies(self):
        logger.info('获取cookies')
        cookies = self.d.get_cookies()
        logger.info(cookies)
        return cookies

    def get_cookie(self, name):
        logger.info(f'获取cookie: {name}')
        cookie = self.d.get_cookie(name)
        logger.info(cookie)
        return cookie

    def delete_all_cookies(self):
        logger.info('删除所有cookie')
        self.d.delete_all_cookies()

    def delete_cookie(self, name):
        logger.info(f'删除cookie: {name}')
        self.d.delete_cookie(name)

    def refresh(self):
        logger.info(f'刷新当前页')
        self.d.refresh()

    def get_title(self):
        logger.info('获取页面标题')
        title = self.d.title
        logger.info(title)
        return title

    def get_url(self):
        logger.info('获取页面url')
        url = self.d.current_url
        logger.info(url)
        return url

    def get_alert_text(self):
        logger.info('获取alert的文本')
        alert_text = self.d.switch_to.alert.text
        logger.info(alert_text)
        return alert_text

    def accept_alert(self):
        logger.info('同意确认框')
        self.d.switch_to.alert.accept()

    def dismiss_alert(self):
        logger.info('取消确认框')
        self.d.switch_to.alert.dismiss()





import os.path
import sys

run_content_android = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='android',
        device_id='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp',
    )
"""

run_content_ios = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='ios',
        device_id='00008101-000E646A3C29003A',
        pkg_name='com.qizhidao.company'
    )
"""

run_content_web = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='web',
        base_url='https://www-pre.qizhidao.com',
    )
"""

run_content_api = """import qrunner


if __name__ == '__main__':
    qrunner.main(
        platform='api',
        base_url='https://www-pre.qizhidao.com',
    )
"""

case_content_android = """import qrunner
from qrunner import story, title


class HomePage(qrunner.Page):
    LOC_AD_CLOSE = {'id_': 'id/bottom_btn', 'desc': '首页广告关闭按钮'}
    LOC_MY = {'id_': 'id/bottom_view', 'index': 3, 'desc': '首页底部我的入口'}
    
    def go_my(self):
        self.elem(self.LOC_AD_CLOSE).click()
        self.elem(self.LOC_MY).click()


@story('首页')
class TestClass(qrunner.TestCase):
    
    def start(self):
        self.hp = HomePage(self.driver)
    
    @title('从首页进入我的页')
    def testcase(self):
        self.start_app()
        self.hp.go_my()
        self.assertText('我的订单')
        self.stop_app()


if __name__ == '__main__':
    qrunner.main(
        platform='android',
        device_id='UJK0220521066836',
        pkg_name='com.qizhidao.clientapp'
    )
"""

case_content_ios = """import qrunner
from qrunner import story, title


class HomePage(qrunner.Page):
    LOC_AD_CLOSE = {'label': 'close white big', 'desc': '首页广告关闭按钮'}
    LOC_MY = {'label': '我的', 'desc': '首页底部我的入口'}
    
    def go_my(self):
        self.elem(self.LOC_AD_CLOSE).click()
        self.elem(self.LOC_MY).click()


@story('首页')
class TestClass(qrunner.TestCase):

    def start(self):
        self.hp = HomePage(self.driver)

    @title('从首页进入我的页')
    def testcase(self):
        self.start_app()
        self.hp.go_my()
        self.assertText('我的订单')
        self.stop_app()


if __name__ == '__main__':
    qrunner.main(
        platform='ios',
        device_id='00008101-000E646A3C29003A',
        pkg_name='com.qizhidao.company'
    )
"""

case_content_web = """import qrunner
from qrunner import story, title


class PatentPage(qrunner.Page):
    url = None
    LOC_SEARCH_INPUT = {'id_': 'driver-home-step1', 'desc': '查专利首页输入框'}
    LOC_SEARCH_SUBMIT = {'id_': 'driver-home-step2', 'desc': '查专利首页搜索确认按钮'}
    
    def simple_search(self):
        self.elem(self.LOC_SEARCH_INPUT).set_text('无人机')
        self.elem(self.LOC_SEARCH_SUBMIT).click()


@story('专利检索')
class TestClass(qrunner.TestCase):
    
    def start(self):
        self.pp = PatentPage(self.driver)
    
    @title('专利简单检索')
    def testcase(self):
        self.pp.open()
        self.pp.simple_search()
        self.assertTitle('无人机专利检索-企知道')


if __name__ == '__main__':
    qrunner.main(
        platform='web',
        base_url='https://patents.qizhidao.com/'
    )
"""

case_content_api = """import qrunner
from qrunner import title, file_data, story


@story('PC站首页')
class TestClass(qrunner.TestCase):

    @title('查询PC站首页banner列表')
    @file_data('card_type', 'data.json')
    def test_getToolCardListForPc(self, card_type):
        path = '/api/qzd-bff-app/qzd/v1/home/getToolCardListForPc'
        payload = {"type": card_type}
        self.post(path, json=payload)
        self.assertEq('code', 0)


if __name__ == '__main__':
    qrunner.main(
        platform='api',
        base_url='https://www.qizhidao.com'
    )
"""

data_content = """{
  "card_type": [0, 1, 2]
}
"""


def create_scaffold(project_name, platform):
    """ create scaffold with specified project name.
    """

    def create_folder(path):
        os.makedirs(path)
        msg = f"created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"created file: {path}"
        print(msg)

    create_folder(project_name)
    # 新增测试用例目录
    create_folder(os.path.join(project_name, 'test_dir'))
    create_file(
        os.path.join(project_name, 'test_dir', "__init__.py"),
        '',
    )
    # 新增测试数据目录
    create_folder(os.path.join(project_name, 'test_data'))
    create_file(
        os.path.join(project_name, 'test_data', "data.json"),
        data_content,
    )
    if platform == 'android':
        # 新增框架入口程序
        create_file(
            os.path.join(project_name, "run.py"),
            run_content_android,
        )
        # 新增安卓测试用例
        create_file(
            os.path.join(project_name, 'test_dir', "test_android.py"),
            case_content_android,
        )

    elif platform == 'ios':
        # 新增框架入口程序
        create_file(
            os.path.join(project_name, "run.py"),
            run_content_ios,
        )
        # 新增ios测试用例
        create_file(
            os.path.join(project_name, 'test_dir', "test_ios.py"),
            case_content_ios,
        )
    elif platform == 'web':
        # 新增框架入口程序
        create_file(
            os.path.join(project_name, "run.py"),
            run_content_web,
        )
        # 新增web测试用例
        create_file(
            os.path.join(project_name, 'test_dir', "test_web.py"),
            case_content_web,
        )
    elif platform == 'api':
        # 新增框架入口程序
        create_file(
            os.path.join(project_name, "run.py"),
            run_content_api,
        )
        # 新增接口测试用例
        create_file(
            os.path.join(project_name, 'test_dir', "test_api.py"),
            case_content_api,
        )
    else:
        print('请输入正确的平台: android、ios、web、api')
        sys.exit()






import inspect
# import time
import typing
# from multiprocessing.dummy import Pool as ThreadPool
# # from typing import Union
from uiautomator2 import UiObject
from uiautomator2.xpath import XPathSelector
from qrunner.utils.log import logger
from qrunner.utils.config import config
from qrunner.core.android.driver import AndroidDriver
from qrunner.utils.exceptions import NoSuchElementException, \
    ElementNameEmptyException


class AndroidElement(object):
    """
    安卓元素定义
    """
    def __init__(self,
                 driver=None,
                 id_=None,
                 class_name=None,
                 text=None,
                 xpath=None,
                 desc=None,
                 index=None):
        """
        param rid: resourceId,
        param cname: className,
        param text,
        param xpath,
        param desc：控件名称,
        param index: 控件索引
        """
        # 参数处理
        kwargs = {}
        self._pkg_name = config.get_pkg()
        if id_ is not None:
            kwargs['resourceId'] = f'{self._pkg_name}:{id_}'
        if class_name is not None:
            kwargs['className'] = class_name
        if text is not None:
            kwargs['text'] = text
        self._xpath = xpath
        self.desc = desc
        if self.desc is None:
            raise ElementNameEmptyException('请设置控件名称')
        self._index = index if index is not None else 0
        self._kwargs = kwargs
        self._driver = driver

        # # 驱动初始化
        # _serial_no = config.get_item('android', 'serial_no')
        # self._driver = AndroidDriver.get_instance(_serial_no)
        # self._d = self._driver.d

    # def find_element(self, retry=3, timeout=3):
    #     """class_name
    #     循环查找元素，查找失败先处理弹窗后重试，后面再考虑xpath要不要用.all()改造一下
    #     @param retry: 重试次数
    #     @param timeout: 每次查找超时时间
    #     @param alert_check: 是否监控弹窗
    #     @return: 找到的元素列表
    #     """
    #     logger.info(f'查找元素: {self._kwargs},{self._index}')
    #     _element = self._d.xpath(self._xpath) if \
    #         self._xpath else self._d(**self._kwargs)[self._index]
    #     while not _element.wait(timeout=timeout):
    #         if retry > 0:
    #             retry -= 1
    #             logger.warning(f'重试 查找元素： {self._kwargs},{self._index}')
    #         else:
    #             frame = inspect.currentframe().f_back
    #             caller = inspect.getframeinfo(frame)
    #             logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
    #             return None
    #     return _element

    def get_driver(self):
        # driver初始化
        _serial_no = config.get_device()
        if self._driver is not None:
            driver = self._driver
        else:
            driver = AndroidDriver.get_instance(_serial_no)
        return driver

    def find_element(self, retry=3, timeout=3, alert_list=None):
        """
        循环查找元素，查找失败先处理弹窗后重试，后面再考虑xpath要不要用.all()改造一下
        @param retry: 重试次数
        @param timeout: 每次查找超时时间
        @param alert_list
        @return: 找到的元素列表
        """

        def handle_alert(driver, alert_list):
            """
            根据不同定位方式进行点击
            @return:
            """

            def click_alert(loc):
                if 'id/' in loc:
                    element = driver(resourceId=f'{self._pkg_name}:{loc}')
                elif '//' in loc:
                    element = driver.xpath(loc)
                else:
                    element = driver(text=loc)
                element.click_exists(timeout=1)

            # 多线程执行点击过程
            # alert_list = config.get_item('app', 'android_alert')
            # pool = ThreadPool(len(alert_list))
            # if alert_list:
            #     pool.map(click_alert, alert_list)
            if alert_list:
                print(f'处理弹窗: {alert_list}')
                for alert in alert_list:
                    click_alert(alert)
                logger.debug('处理异常弹窗完成')

        driver = self.get_driver()

        logger.info(f'查找元素: {self._kwargs},{self._index}')
        _element = driver.d.xpath(self._xpath) if \
            self._xpath else driver.d(**self._kwargs)[self._index]
        handle_alert(driver.d, alert_list)  # 处理异常弹窗
        cur_retry = retry
        while not _element.wait(timeout=timeout):
            if cur_retry > 0:
                logger.warning(f'第{retry-cur_retry+1}次重试，查找元素： {self._kwargs}')
                cur_retry -= 1
                handle_alert(driver.d, alert_list)  # 处理异常弹窗
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
                return None
        _elements = driver.d.xpath(self._xpath).all() if \
            self._xpath else driver.d(**self._kwargs)
        return _elements

    # def find_elements(self, retry=3, timeout=3, alert_list=None):
    #     """
    #     循环查找元素，查找失败先处理弹窗后重试，后面再考虑xpath要不要用.all()改造一下
    #     @param retry: 重试次数
    #     @param timeout: 每次查找超时时间
    #     @param alert_list
    #     @return: 找到的元素列表
    #     """
    #     logger.info(f'查找元素列表: {self._kwargs}')
    #     if not self._xpath:
    #         _elements = self._d(**self._kwargs)
    #     else:
    #         raise AssertionError('find_elements方法不支持xpath定位')
    #     self.handle_alert(alert_list)  # 处理异常弹窗
    #     cur_retry = retry
    #     while not _elements.wait(timeout=timeout):
    #         if cur_retry > 0:
    #             logger.warning(f'第{retry-cur_retry+1}次重试，查找元素： {self._kwargs}')
    #             cur_retry -= 1
    #             self.handle_alert(alert_list)  # 处理异常弹窗
    #         else:
    #             frame = inspect.currentframe().f_back
    #             caller = inspect.getframeinfo(frame)
    #             logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
    #             return None
    #     return _elements

    def get_elements(self, retry=3, timeout=3, alert_list=None):
        """
        获取元素列表
        @param retry:
        @param timeout:
        @param alert_list
        @return:
        """
        driver = self.get_driver()
        elements = self.find_element(retry=retry, timeout=timeout, alert_list=alert_list)
        if elements is None:
            driver.screenshot(f'[控件 {self.desc} 定位失败]')
            raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
        else:
            driver.screenshot(self.desc)
        return elements

    # def __getitem__(self, index):
    #     elements = self.get_elements()
    #     return elements[index]

    def get_element(self, retry=3, timeout=3, alert_list=None):
        """
        获取指定一个元素
        @param retry:
        @param timeout:
        @param alert_list
        @return:
        """
        # element = self.find_element(retry=retry, timeout=timeout, alert_list=alert_list)
        # if element is None:
        #     self._driver.screenshot(f'[控件 {self.desc} 定位失败]')
        #     raise NoSuchElementException(f'[控件 {self.desc} 定位失败]')
        elements = self.get_elements(retry=retry, timeout=timeout, alert_list=alert_list)
        return elements[self._index]

    @property
    def info(self):
        """获取元素信息"""
        logger.info(f'获取元素: {self._kwargs} 的所有信息')
        return self.get_element().info

    @property
    def text(self):
        """获取元素文本属性"""
        logger.info(f'获取元素: {self._kwargs} 的文本')
        return self.get_element().info.get('text')

    @property
    def bounds(self):
        """获取元素坐标"""
        logger.info(f'获取元素: {self._kwargs} 的坐标')
        return self.get_element().info.get('bounds')

    @property
    def rect(self):
        """获取元素左上角的坐标以及宽高"""
        logger.info(f'获取元素: {self._kwargs} 左上角的坐标以及宽高')
        bounds = self.get_element().info.get('bounds')
        x = bounds['left']
        y = bounds['top']
        width = bounds['right'] - x
        height = bounds['bottom'] - y
        return [x, y, width, height]

    @property
    def visibleBounds(self):
        """获取元素可见坐标"""
        logger.info(f'获取元素: {self._kwargs} 的可见坐标')
        return self.get_element().info.get('visibleBounds')

    @property
    def focusable(self):
        """获取元素是否聚焦"""
        logger.info(f'获取元素: {self._kwargs} 是否聚焦')
        return self.get_element().info.get('focusable')

    @property
    def selected(self):
        """获取元素是否选中"""
        logger.info(f'获取元素: {self._kwargs} 是否选中')
        return self.get_element().info.get('selected')

    def child(self, *args, **kwargs):
        """获取元素儿子节点，不能用于PageObject，会导致在应用启动前进行元素识别"""
        logger.info(f'获取元素 {self._kwargs},{self._index} 的子元素{kwargs}')
        return self.get_element().child(*args, **kwargs)

    def brother(self, *args, **kwargs):
        """获取兄弟元素"""
        logger.info(f'获取元素 {self._kwargs},{self._index} 的兄弟元素{kwargs}')
        return self.get_element().sibling(*args, **kwargs)

    def left(self, *args, **kwargs):
        """获取左边元素"""
        logger.info(f'获取元素 {self._kwargs} 左边的元素 {kwargs}')
        return self.get_element().left(*args, **kwargs)

    def right(self, *args, **kwargs):
        """获取右边元素"""
        logger.info(f'获取元素 {self._kwargs} 右边的元素 {kwargs}')
        return self.get_element().right(*args, **kwargs)

    def up(self, *args, **kwargs):
        """获取上边的元素"""
        logger.info(f'获取元素 {self._kwargs} 上边的元素 {kwargs}')
        return self.get_element().up(*args, **kwargs)

    def down(self, *args, **kwargs):
        """获取下边的元素"""
        logger.info(f'获取元素 {self._kwargs} 下边的元素 {kwargs}')
        return self.get_element().down(*args, **kwargs)

    def exists(self, timeout=1):
        """判断元素是否存在"""
        logger.info(f'判断元素是否存在: {self._kwargs},{self._index}')
        element = self.find_element(retry=0, timeout=timeout)
        if element is None:
            # self._driver.screenshot(f'元素定位失败')
            return False
        return True

    @staticmethod
    def _adapt_center(e: typing.Union[UiObject, XPathSelector], offset=(0.5, 0.5)):
        if isinstance(e, UiObject):
            return e.center(offset=offset)
        else:
            return e.offset(offset[0], offset[1])

    def click(self, retry=3, timeout=3, alert_list=None):
        """
        @param: retry，重试次数
        @param: timeout，每次重试超时时间
        @param: alert_list，异常弹窗列表
        """
        logger.info(f'点击元素: {self._kwargs},{self._index}')
        element = self.get_element(retry=retry, timeout=timeout, alert_list=alert_list)
        # 这种方式经常点击不成功，感觉是页面刷新有影响
        # element.click()
        x, y = self._adapt_center(element)
        self.get_driver().d.click(x, y)
        logger.debug('点击成功')

    # def click_no_retry(self, offset=(0.5, 0.5), timeout=1):
    #     logger.info(f'点击元素: {self._kwargs},{self._index}')
    #     element = self.get_element(timeout=timeout)
    #     x, y = self._adapt_center(element, offset=offset)
    #     self._d.click(x, y)
    #     logger.debug('点击成功')

    def click_exists(self, timeout=3):
        """元素存在才点击"""
        logger.info(f'存在才点击元素: {self._kwargs},{self._index}')
        if self.exists(timeout=timeout):
            self.click()

    def click_gone(self):
        """等元素消失后再点击"""
        logger.info(f'等元素 {self._kwargs} 消失后再点击')
        flag = self.get_element().click_gone()
        logger.info(flag)
        return flag

    def wait_gone(self, timeout=3):
        """等待元素消失"""
        logger.info(f'等元素 {self._kwargs} 消失')
        flag = self.get_element().wait_gone(timeout=timeout)
        logger.info(flag)
        return flag

    def long_click(self):
        """长按"""
        logger.info(f'长按元素 {self._kwargs}')
        self.get_element().long_click()

    def set_text(self, text):
        """输入文本"""
        logger.info(f'输入文本: {text}')
        self.get_element().set_text(text)

    def clear_text(self):
        """清除文本"""
        logger.info('清除文本')
        self.get_element().clear_text()

    def drag_to(self, *args, **kwargs):
        """拖动到另外一个元素的位置"""
        logger.info(f'从当前元素{self._kwargs},{self._index}, 拖动到元素: {kwargs}')
        self.get_element().drag_to(*args, **kwargs)

    def swipe_left(self):
        """向左滑动"""
        logger.info(f'往左滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("left")

    def swipe_right(self):
        """向右滑动"""
        logger.info(f'往右滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("right")

    def swipe_up(self):
        """向上滑动"""
        logger.info(f'往上滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("up")

    def swipe_down(self):
        """向下滑动"""
        logger.info(f'往下滑动元素: {self._kwargs},{self._index}')
        self.get_element().swipe("down")






import qrunner
from qrunner import title, file_data, story


@story('PC站首页')
class TestClass(qrunner.TestCase):

    @title('查询PC站首页banner列表')
    @file_data('card_type', 'data.json')
    def test_getToolCardListForPc(self, card_type):
        path = '/api/qzd-bff-app/qzd/v1/home/getToolCardListForPc'
        payload = {"type": card_type}
        self.post(path, json=payload)
        self.assertEq('code', 0)


if __name__ == '__main__':
    qrunner.main(
        platform='api',
        base_url='https://www-pre.qizhidao.com'
    )



import os
import yaml


local_path = os.path.dirname(os.path.realpath(__file__))
root_path = os.path.dirname(local_path)


class Config:
    def __init__(self):
        self.file_path = os.path.join(root_path, 'running', 'conf.yml')
        with open(self.file_path, "r", encoding="utf-8") as f:
            self.yaml_data = yaml.load(f.read(), Loader=yaml.FullLoader)

    def get(self, module, key):
        return self.yaml_data[module][key]

    def set(self, module, key, value):
        self.yaml_data[module][key] = value
        with open(self.file_path, 'w', encoding="utf-8") as f:
            yaml.dump(self.yaml_data, f)

    def get_platform(self):
        return self.get('common', 'platform')

    def get_device(self):
        return self.get('app', 'device_id')

    def get_pkg(self):
        return self.get('app', 'pkg_name')

    def get_browser(self):
        return self.get('web', 'browser')

    def get_host(self):
        return self.get('common', 'base_url')

    def get_login(self):
        return self.get('common', 'login_headers')

    def get_visit(self):
        return self.get('common', 'visit_headers')

    def get_timeout(self):
        return self.get('common', 'timeout')


config = Config()


if __name__ == '__main__':
    pass








import os
import socket


def get_device_list():
    """获取当前连接的设备列表"""
    cmd = 'tidevice list'
    output = os.popen(cmd).read()
    device_list = [item.split(' ')[0] for item in output.split('\n') if item]
    return device_list


def get_tcp_port(udid: str):
    """获取可用端口号"""
    # tcp = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    # tcp.bind(("", 0))
    # _, port = tcp.getsockname()
    # tcp.close()
    port = int(udid.split('-')[0])
    return port


if __name__ == '__main__':
    print(get_device_list())




import sys
import time
from qrunner.utils.log import logger
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.core.web.driver import WebDriver
from qrunner.core.web.element import WebElement
from qrunner.core.h5.driver import H5Driver
from typing import Union


class Page(object):
    """页面基类，用于pom模式封装"""

    url = None

    def __init__(self, driver: Union[AndroidDriver, IosDriver,
                                     WebDriver, H5Driver]):
        self.driver = driver

    @staticmethod
    def sleep(n):
        """休眠"""
        logger.info(f'休眠 {n} 秒')
        time.sleep(n)

    def open(self):
        """打开页面"""
        try:
            self.driver.open_url(self.url)
        except Exception as e:
            logger.error(f'请设置页面url: {str(e)}')
            sys.exit()

    def elem(self, loc_dict: dict):
        """页面元素封装"""
        if isinstance(self.driver, AndroidDriver):
            return AndroidElement(**loc_dict)
        elif isinstance(self.driver, WebDriver):
            return WebElement(**loc_dict)
        elif isinstance(self.driver, IosDriver):
            return IosElement(**loc_dict)






from qrunner.running.runner import main
from qrunner.utils.log import logger
from qrunner.utils.decorate import *
from qrunner.utils.config import config
from qrunner.core.android.driver import AndroidDriver
from qrunner.core.android.element import AndroidElement
from qrunner.core.h5.driver import H5Driver
from qrunner.core.ios.driver import IosDriver
from qrunner.core.ios.element import IosElement
from qrunner.core.web.driver import WebDriver
from qrunner.core.web.element import WebElement
from qrunner.core.web.driver import ChromeConfig, IEConfig, \
    FirefoxConfig, EdgeConfig, SafariConfig
from qrunner.core.api.request import HttpRequest
from qrunner.case import TestCase
from qrunner.page import Page
from qrunner.utils.mail import Mail
from qrunner.utils.dingtalk import DingTalk

__version__ = "0.1.0"
__description__ = "全平台自动化测试框架"



import inspect
import os
import pytest
from qrunner.utils.log import logger
from qrunner.utils.config import config


class TestMain(object):
    """
    Support for app、web、http
    """
    def __init__(self,
                 platform: str = None,
                 device_id: str = None,
                 pkg_name: str = None,
                 browser: str = 'chrome',
                 case_path: str = None,
                 rerun: int = 0,
                 concurrent: bool = False,
                 base_url: str = None,
                 headers: dict = None,
                 timeout: int = 10,
                 ):
        """

        @param platform: 平台，android、ios、web、api
        @param device_id: 设备id，针对安卓和ios
        @param pkg_name: 应用包名，针对安卓和ios
        @param browser: 浏览器类型，chrome、firefox、edge、safari
        @param case_path: 用例目录，默认代表当前文件、.代表当前目录
        @param rerun: 失败重试次数
        @param concurrent: 是否并发执行，针对接口
        @param base_url: 域名，针对接口和web
        @param headers: 登录和游客请求头，针对接口和web，格式: {
            "login_headers": {},
            "visit_headers": {}
        }
        @param timeout: 超时时间，针对接口和web
        """
        # 将数据写入全局变量
        config.set('common', 'platform', platform)
        config.set('app', 'device_id', device_id)
        config.set('app', 'pkg_name', pkg_name)
        config.set('web', 'browser', browser)
        config.set('common', 'base_url', base_url)
        if headers is not None:
            login_headers = headers.pop('login_headers', {})
            config.set('common', 'login_headers', login_headers)
            visit_headers = headers.pop('visit_headers', {})
            config.set('common', 'visit_headers', visit_headers)
        config.set('common', 'timeout', timeout)

        # 执行用例
        logger.info('执行用例')
        if case_path is None:
            stack_t = inspect.stack()
            ins = inspect.getframeinfo(stack_t[1][0])
            file_dir = os.path.dirname(os.path.abspath(ins.filename))
            file_path = ins.filename
            if "\\" in file_path:
                this_file = file_path.split("\\")[-1]
            elif "/" in file_path:
                this_file = file_path.split("/")[-1]
            else:
                this_file = file_path
            case_path = os.path.join(file_dir, this_file)
        logger.info(f'用例路径: {case_path}')
        cmd_list = [
            '-sv',
            '--reruns', str(rerun),
            '--alluredir', 'allure-results', '--clean-alluredir'
        ]
        if case_path:
            cmd_list.insert(0, case_path)
        if concurrent:
            """仅支持http接口测试"""
            cmd_list.insert(1, '-n')
            cmd_list.insert(2, 'auto')
            cmd_list.insert(3, '--dist=loadscope')
        logger.info(cmd_list)
        pytest.main(cmd_list)


main = TestMain


if __name__ == '__main__':
    main()




#! usr/bin/python
# -*- coding:utf-8 -*-
class BaseError(Exception):
    def __init__(self, message="", *args, **kwargs):
        self.message = message

    def __repr__(self):
        return repr(self.message)


class NoModuleError(BaseError):
    """ Missing dependent module """


class CreateExtractorError(BaseError):
    """ An error occurred while create Extractor """


class NoEnoughPointsError(BaseError):
    """ detect not enough feature points in input images"""


class CudaSurfInputImageError(BaseError):
    """ The image size does not conform to CUDA standard """
    # https://stackoverflow.com/questions/42492060/surf-cuda-error-while-computing-descriptors-and-keypoints
    # https://github.com/opencv/opencv_contrib/blob/master/modules/xfeatures2d/src/surf.cuda.cpp#L151


class CudaOrbDetectorError(BaseError):
    """ An CvError when orb detector error occurred """


class HomographyError(BaseError):
    """ An error occurred while findHomography """


class MatchResultError(BaseError):
    """ An error occurred while result out of screen"""


class PerspectiveTransformError(BaseError):
    """ An error occurred while perspectiveTransform """


class InputImageError(BaseError):
    """ An error occurred while input image place/dtype/channels error"""



import inspect
from typing import Union
from wda import Selector
from qrunner.utils.log import logger
from qrunner.utils.config import config
from qrunner.core.ios.driver import IosDriver
from qrunner.utils.exceptions import NoSuchElementException, \
    ElementNameEmptyException


class IosElement(object):
    """
    IOS原生元素定义
    """
    def __init__(self,
                 driver=None,
                 name=None,
                 label=None,
                 value=None,
                 text=None,
                 class_name=None,
                 xpath=None,
                 index=None,
                 desc=None):
        """
        param name,
        param label,
        param value,
        param text,
        param cname：className,
        param xpath,
        param index: 索引,
        param desc: 控件名称
        """
        # 参数处理
        kwargs = {}
        if name is not None:
            kwargs['name'] = name
        if label is not None:
            kwargs['label'] = label
        if value is not None:
            kwargs['value'] = value
        if text is not None:
            kwargs['text'] = text
        if class_name is not None:
            kwargs['className'] = class_name
        self._xpath = xpath
        self._index = index if index is not None else 0
        self.desc = desc
        if self.desc is None:
            raise ElementNameEmptyException('请设置控件名称')
        self._kwargs = kwargs
        self._driver = None

        # # 驱动初始化
        # _serial_no = config.get_item('ios', 'serial_no')
        # self._driver = IosDriver.get_instance(_serial_no)
        # self._d = self._driver.d

    def get_driver(self):
        if self._driver is not None :
            driver = self._driver
        else:
            serial_no = config.get_device()
            driver = IosDriver.get_instance(serial_no)
        return driver

    # def _find_element(self, retry=3, timeout=3):
    #     """
    #     循环查找元素，查找失败先处理弹窗后重试
    #     @param retry: 重试次数
    #     @param timeout: 单次查找超时时间
    #     @return:
    #     """
    #     self._element = self._d.xpath(self._xpath) if \
    #         self._xpath else self._d(**self._kwargs)[self._index]
    #     self.handle_alert()
    #     while not self._element.wait(timeout=timeout):
    #         if retry > 0:
    #             retry -= 1
    #             logger.warning(f'重试 查找元素： {self._kwargs}')
    #             self.handle_alert()
    #         else:
    #             frame = inspect.currentframe().f_back
    #             caller = inspect.getframeinfo(frame)
    #             logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
    #             return None
    #     return self._element

    def _find_element(self, retry=3, timeout=3, alert_list=None):
        """
        循环查找元素，查找失败先处理弹窗后重试
        @param retry: 重试次数
        @param timeout: 单次查找超时时间
        @return:
        """

        # 多线程处理异常弹窗
        def handle_alert(driver, alert_list):
            """
            根据不同定位方式进行点击
            @return:
            """

            def click_alert(loc):
                if '//' in loc:
                    element = driver.d.xpath(loc)
                else:
                    element = driver.d(label=loc)
                element.click_exists()

            # 多线程存在问题，目前没有使用多线程
            if alert_list:
                print(f'处理弹窗: {alert_list}')
                for alert in alert_list:
                    click_alert(alert)

        # 驱动初始化
        driver = self.get_driver()

        element = driver.d.xpath(self._xpath) if \
            self._xpath else driver.d(**self._kwargs)[self._index]
        handle_alert(driver, alert_list)
        cur_retry = retry
        while not element.wait(timeout=timeout):
            if cur_retry > 0:
                logger.warning(f'第{retry-cur_retry+1}次重试，查找元素： {self._kwargs},{self._index}')
                cur_retry -= 1
                handle_alert(driver, alert_list)
            else:
                frame = inspect.currentframe().f_back
                caller = inspect.getframeinfo(frame)
                logger.warning(f'【{caller.function}:{caller.lineno}】未找到元素 {self._kwargs}')
                return None
        elements = driver.d.xpath(self._xpath).find_elements() if \
            self._xpath else driver.d(**self._kwargs)
        return elements

    def get_elements(self, retry=3, timeout=3, alert_list=None):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param retry:
        @param timeout:
        @param alert_list
        @return:
        """
        driver = self.get_driver()

        elements: Union[Selector] = self._find_element(retry=retry, timeout=timeout, alert_list=alert_list)
        if elements is None:
            driver.screenshot(f'[控件 {self.desc}] 定位失败')
            raise NoSuchElementException(f'[控件 {self.desc}] 定位失败')
        else:
            driver.screenshot(self.desc)
        return elements

    # def __getitem__(self, index):
    #     elements = self.get_elements()
    #     return elements[index]

    def get_element(self, retry=3, timeout=3, alert_list=None):
        """
        针对元素定位失败的情况，抛出NoSuchElementException异常
        @param retry:
        @param timeout:
        @param alert_list
        @return:
        """
        elements = self.get_elements(retry=retry, timeout=timeout, alert_list=alert_list)
        return elements[self._index]

    @property
    def info(self):
        """获取元素信息"""
        logger.info(f'获取元素: {self._kwargs} 的所有属性')
        return self.get_element().info

    @property
    def text(self):
        """获取元素文本"""
        logger.info(f'获取元素: {self._kwargs} 的text属性')
        return self.get_element().text

    @property
    def className(self):
        """获取元素className"""
        logger.info(f'获取元素: {self._kwargs} 的className属性')
        return self.get_element().className

    @property
    def name(self):
        """获取元素name"""
        logger.info(f'获取元素: {self._kwargs} 的name属性')
        return self.get_element().name

    @property
    def visible(self):
        """获取元素visible属性"""
        logger.info(f'获取元素: {self._kwargs} 的visible属性')
        return self.get_element().visible

    @property
    def value(self):
        """获取元素value"""
        logger.info(f'获取元素: {self._kwargs} 的value属性')
        return self.get_element().value

    @property
    def label(self):
        """获取元素label"""
        logger.info(f'获取元素: {self._kwargs} 的label属性')
        return self.get_element().label

    @property
    def enabled(self):
        """获取元素enabled属性"""
        logger.info(f'获取元素: {self._kwargs} 的enabled属性')
        return self.get_element().enabled

    @property
    def displayed(self):
        """获取元素displayed属性"""
        logger.info(f'获取元素: {self._kwargs} 的displayed属性')
        return self.get_element().displayed

    @property
    def bounds(self):
        """获取元素bounds属性"""
        logger.info(f'获取元素: {self._kwargs} 的bounds属性')
        return self.get_element().bounds

    @property
    def rect(self):
        """获取元素左上角坐标和宽高"""
        logger.info(f'获取元素: {self._kwargs} 的左上角坐标和宽高')
        return [item * self.get_driver().d.scale for item in list(self.get_element().bounds)]

    def exists(self, timeout=1):
        """
        判断元素是否存在当前页面
        @param timeout:
        @return:
        """
        logger.info(f'元素 {self._kwargs},{self._index} 是否存在:')
        element = self._find_element(retry=0, timeout=timeout)
        if element is None:
            # self._driver.screenshot(f'元素定位失败')
            return False
        return True

    def wait_gone(self, timeout=10):
        """等待元素消失"""
        logger.info(f'等待元素{self._kwargs}消失')
        flag = self.get_element().wait_gone(timeout=timeout)
        logger.info(flag)
        return flag

    def click(self, retry=3, timeout=3, alert_list=None):
        """
        单击
        @param: retry，重试次数
        @param: timeout，每次重试超时时间
        @param: alert_list，异常弹窗列表
        """
        logger.info(f'点击元素: {self._kwargs},{self._index}')
        self.get_element(retry=retry, timeout=timeout, alert_list=alert_list).click()

    # def click_no_retry(self):
    #     logger.info(f'点击元素: {self._kwargs},{self._index}')
    #     self.get_element(retry=0, timeout=1).click()

    def click_exists(self, timeout=3):
        """元素存在时点击"""
        logger.info(f'存在才点击元素: {self._kwargs},{self._index}')
        if self.exists(timeout=timeout):
            self.click()

    def clear_text(self):
        """清除文本"""
        logger.info('清除文本: {text}')
        self.get_element().clear_text()

    def set_text(self, text):
        """输入内容"""
        logger.info(f'输入框 {self._kwargs},{self._index} 输入: {text}')
        self.get_element().set_text(text)

    def scroll(self, direction=None):
        """
        scroll to make element visiable
        @param: direction，方向，"up", "down", "left", "right"
        @return:
        """
        if direction is not None:
            self.get_element().scroll(direction=direction)
        else:
            self.get_element().scroll()

    def swipe_left(self):
        """往左滑动"""
        self.get_element().swipe("left")

    def swipe_right(self):
        """往右滑动"""
        self.get_element().swipe("right")

    def swipe_up(self):
        """往上滑动"""
        self.get_element().swipe("up")

    def swipe_down(self):
        """往下滑动"""
        self.get_element().swipe("down")

    def child(self, *args, **kwargs):
        """获取兄弟节点，不能用于PageObject，会导致在应用启动前进行元素识别"""
        return self.get_element().child(*args, **kwargs)






